#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017 Google
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
# ----------------------------------------------------------------------------
#
#     ***     AUTO GENERATED CODE    ***    AUTO GENERATED CODE     ***
#
# ----------------------------------------------------------------------------
#
#     This file is automatically generated by Magic Modules and manual
#     changes will be clobbered when the file is regenerated.
#
#     Please read more about how to change this file at
#     https://www.github.com/GoogleCloudPlatform/magic-modules
#
# ----------------------------------------------------------------------------

from __future__ import absolute_import, division, print_function
__metaclass__ = type

################################################################################
# Documentation
################################################################################

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ["preview"],
                    'supported_by': 'community'}

DOCUMENTATION = '''
---
module: gcp_compute_instance
description:
    - An instance is a virtual machine (VM) hosted on Google's infrastructure.
short_description: Creates a GCP Instance
version_added: 2.6
author: Google Inc. (@googlecloudplatform)
requirements:
    - python >= 2.6
    - requests >= 2.18.4
    - google-auth >= 1.3.0
options:
    state:
        description:
            - Whether the given object should exist in GCP
        choices: ['present', 'absent']
        default: 'present'
    can_ip_forward:
        description:
            - Allows this instance to send and receive packets with non-matching destination or
              source IPs. This is required if you plan to use this instance to forward routes.
        required: false
        type: bool
    disks:
        description:
            - An array of disks that are associated with the instances that are created from this
              template.
        required: false
        suboptions:
            auto_delete:
                description:
                    - Specifies whether the disk will be auto-deleted when the instance is deleted (but
                      not when the disk is detached from the instance).
                    - 'Tip: Disks should be set to autoDelete=true so that leftover disks are not left
                      behind on machine deletion.'
                required: false
                type: bool
            boot:
                description:
                    - Indicates that this is a boot disk. The virtual machine will use the first partition
                      of the disk for its root filesystem.
                required: false
                type: bool
            device_name:
                description:
                    - Specifies a unique device name of your choice that is reflected into the /dev/disk/by-id/google-*
                      tree of a Linux operating system running within the instance. This name can be used
                      to reference the device for mounting, resizing, and so on, from within the instance.
                required: false
            disk_encryption_key:
                description:
                    - Encrypts or decrypts a disk using a customer-supplied encryption key.
                required: false
                suboptions:
                    raw_key:
                        description:
                            - Specifies a 256-bit customer-supplied encryption key, encoded in RFC 4648 base64
                              to either encrypt or decrypt this resource.
                        required: false
                    rsa_encrypted_key:
                        description:
                            - Specifies an RFC 4648 base64 encoded, RSA-wrapped 2048-bit customer-supplied encryption
                              key to either encrypt or decrypt this resource.
                        required: false
                    sha256:
                        description:
                            - The RFC 4648 base64 encoded SHA-256 hash of the customer-supplied encryption key
                              that protects this resource.
                        required: false
            index:
                description:
                    - Assigns a zero-based index to this disk, where 0 is reserved for the boot disk.
                      For example, if you have many disks attached to an instance, each disk would have
                      a unique index number. If not specified, the server will choose an appropriate value.
                required: false
            initialize_params:
                description:
                    - Specifies the parameters for a new disk that will be created alongside the new instance.
                      Use initialization parameters to create boot disks or local SSDs attached to the
                      new instance.
                required: false
                suboptions:
                    disk_name:
                        description:
                            - Specifies the disk name. If not specified, the default is to use the name of the
                              instance.
                        required: false
                    disk_size_gb:
                        description:
                            - Specifies the size of the disk in base-2 GB.
                        required: false
                    disk_type:
                        description:
                            - Reference to a gcompute_disk_type resource.
                            - Specifies the disk type to use to create the instance.
                            - If not specified, the default is pd-standard.
                        required: false
                    source_image:
                        description:
                            - The source image to create this disk. When creating a new instance, one of initializeParams.sourceImage
                              or disks.source is required.  To create a disk with one of the public operating
                              system images, specify the image by its family name.
                        required: false
                    source_image_encryption_key:
                        description:
                            - The customer-supplied encryption key of the source image. Required if the source
                              image is protected by a customer-supplied encryption key.
                            - Instance templates do not store customer-supplied encryption keys, so you cannot
                              create disks for instances in a managed instance group if the source images are
                              encrypted with your own keys.
                        required: false
                        suboptions:
                            raw_key:
                                description:
                                    - Specifies a 256-bit customer-supplied encryption key, encoded in RFC 4648 base64
                                      to either encrypt or decrypt this resource.
                                required: false
                            sha256:
                                description:
                                    - The RFC 4648 base64 encoded SHA-256 hash of the customer-supplied encryption key
                                      that protects this resource.
                                required: false
            interface:
                description:
                    - Specifies the disk interface to use for attaching this disk, which is either SCSI
                      or NVME. The default is SCSI.
                    - Persistent disks must always use SCSI and the request will fail if you attempt to
                      attach a persistent disk in any other format than SCSI.
                required: false
                choices: ['SCSI', 'NVME']
            mode:
                description:
                    - The mode in which to attach this disk, either READ_WRITE or READ_ONLY. If not specified,
                      the default is to attach the disk in READ_WRITE mode.
                required: false
                choices: ['READ_WRITE', 'READ_ONLY']
            source:
                description:
                    - Reference to a gcompute_disk resource. When creating a new instance, one of initializeParams.sourceImage
                      or disks.source is required.
                    - If desired, you can also attach existing non-root persistent disks using this property.
                      This field is only applicable for persistent disks.
                required: false
            type:
                description:
                    - Specifies the type of the disk, either SCRATCH or PERSISTENT. If not specified,
                      the default is PERSISTENT.
                required: false
                choices: ['SCRATCH', 'PERSISTENT']
    guest_accelerators:
        description:
            - List of the type and count of accelerator cards attached to the instance .
        required: false
        suboptions:
            accelerator_count:
                description:
                    - The number of the guest accelerator cards exposed to this instance.
                required: false
            accelerator_type:
                description:
                    - Full or partial URL of the accelerator type resource to expose to this instance.
                required: false
    label_fingerprint:
        description:
            - A fingerprint for this request, which is essentially a hash of the metadata's contents
              and used for optimistic locking. The fingerprint is initially generated by Compute
              Engine and changes after every request to modify or update metadata. You must always
              provide an up-to-date fingerprint hash in order to update or change metadata.
        required: false
    metadata:
        description:
            - The metadata key/value pairs to assign to instances that are created from this template.
              These pairs can consist of custom metadata or predefined keys.
        required: false
    machine_type:
        description:
            - A reference to a machine type which defines VM kind.
        required: false
    min_cpu_platform:
        description:
            - Specifies a minimum CPU platform for the VM instance. Applicable values are the
              friendly names of CPU platforms .
        required: false
    name:
        description:
            - The name of the resource, provided by the client when initially creating the resource.
              The resource name must be 1-63 characters long, and comply with RFC1035. Specifically,
              the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?`
              which means the first character must be a lowercase letter, and all following characters
              must be a dash, lowercase letter, or digit, except the last character, which cannot
              be a dash.
        required: false
    network_interfaces:
        description:
            - An array of configurations for this interface. This specifies how this interface
              is configured to interact with other network services, such as connecting to the
              internet. Only one network interface is supported per instance.
        required: false
        suboptions:
            access_configs:
                description:
                    - An array of configurations for this interface. Currently, only one access config,
                      ONE_TO_ONE_NAT, is supported. If there are no accessConfigs specified, then this
                      instance will have no external internet access.
                required: false
                suboptions:
                    name:
                        description:
                            - The name of this access configuration. The default and recommended name is External
                              NAT but you can use any arbitrary string you would like. For example, My external
                              IP or Network Access.
                        required: true
                    nat_ip:
                        description:
                            - Specifies the title of a gcompute_address.
                            - An external IP address associated with this instance.
                            - Specify an unused static external IP address available to the project or leave this
                              field undefined to use an IP from a shared ephemeral IP address pool. If you specify
                              a static external IP address, it must live in the same region as the zone of the
                              instance.
                        required: false
                    type:
                        description:
                            - The type of configuration. The default and only option is ONE_TO_ONE_NAT.
                        required: true
                        choices: ['ONE_TO_ONE_NAT']
            alias_ip_ranges:
                description:
                    - An array of alias IP ranges for this network interface. Can only be specified for
                      network interfaces on subnet-mode networks.
                required: false
                suboptions:
                    ip_cidr_range:
                        description:
                            - The IP CIDR range represented by this alias IP range.
                            - This IP CIDR range must belong to the specified subnetwork and cannot contain IP
                              addresses reserved by system or used by other network interfaces. This range may
                              be a single IP address (e.g. 10.2.3.4), a netmask (e.g. /24) or a CIDR format string
                              (e.g. 10.1.2.0/24).
                        required: false
                    subnetwork_range_name:
                        description:
                            - Optional subnetwork secondary range name specifying the secondary range from which
                              to allocate the IP CIDR range for this alias IP range. If left unspecified, the
                              primary range of the subnetwork will be used.
                        required: false
            name:
                description:
                    - The name of the network interface, generated by the server. For network devices,
                      these are eth0, eth1, etc .
                required: false
            network:
                description:
                    - Specifies the title of an existing gcompute_network.  When creating an instance,
                      if neither the network nor the subnetwork is specified, the default network global/networks/default
                      is used; if the network is not specified but the subnetwork is specified, the network
                      is inferred.
                required: false
            network_ip:
                description:
                    - An IPv4 internal network address to assign to the instance for this network interface.
                      If not specified by the user, an unused internal IP is assigned by the system.
                required: false
            subnetwork:
                description:
                    - Reference to a gcompute_subnetwork resource.
                    - If the network resource is in legacy mode, do not provide this property.  If the
                      network is in auto subnet mode, providing the subnetwork is optional. If the network
                      is in custom subnet mode, then this field should be specified.
                required: false
    scheduling:
        description:
            - Sets the scheduling options for this instance.
        required: false
        suboptions:
            automatic_restart:
                description:
                    - Specifies whether the instance should be automatically restarted if it is terminated
                      by Compute Engine (not terminated by a user).
                    - You can only set the automatic restart option for standard instances. Preemptible
                      instances cannot be automatically restarted.
                required: false
                type: bool
            on_host_maintenance:
                description:
                    - Defines the maintenance behavior for this instance. For standard instances, the
                      default behavior is MIGRATE. For preemptible instances, the default and only possible
                      behavior is TERMINATE.
                    - For more information, see Setting Instance Scheduling Options.
                required: false
            preemptible:
                description:
                    - Defines whether the instance is preemptible. This can only be set during instance
                      creation, it cannot be set or changed after the instance has been created.
                required: false
                type: bool
    service_accounts:
        description:
            - A list of service accounts, with their specified scopes, authorized for this instance.
              Only one service account per VM instance is supported.
        required: false
        suboptions:
            email:
                description:
                    - Email address of the service account.
                required: false
            scopes:
                description:
                    - The list of scopes to be made available for this service account.
                required: false
    tags:
        description:
            - A list of tags to apply to this instance. Tags are used to identify valid sources
              or targets for network firewalls and are specified by the client during instance
              creation. The tags can be later modified by the setTags method. Each tag within
              the list must comply with RFC1035.
        required: false
        suboptions:
            fingerprint:
                description:
                    - Specifies a fingerprint for this request, which is essentially a hash of the metadata's
                      contents and used for optimistic locking.
                    - The fingerprint is initially generated by Compute Engine and changes after every
                      request to modify or update metadata. You must always provide an up-to-date fingerprint
                      hash in order to update or change metadata.
                required: false
            items:
                description:
                    - An array of tags. Each tag must be 1-63 characters long, and comply with RFC1035.
                required: false
    zone:
        description:
            - A reference to the zone where the machine resides.
        required: true
extends_documentation_fragment: gcp
'''

EXAMPLES = '''
- name: create a disk
  gcp_compute_disk:
      name: "disk-instance"
      size_gb: 50
      source_image: projects/ubuntu-os-cloud/global/images/family/ubuntu-1604-lts
      zone: us-central1-a
      project: "{{ gcp_project }}"
      auth_kind: "{{ gcp_cred_kind }}"
      service_account_file: "{{ gcp_cred_file }}"
      state: present
  register: disk

- name: create a network
  gcp_compute_network:
      name: "network-instance"
      project: "{{ gcp_project }}"
      auth_kind: "{{ gcp_cred_kind }}"
      service_account_file: "{{ gcp_cred_file }}"
      state: present
  register: network

- name: create a address
  gcp_compute_address:
      name: "address-instance"
      region: us-central1
      project: "{{ gcp_project }}"
      auth_kind: "{{ gcp_cred_kind }}"
      service_account_file: "{{ gcp_cred_file }}"
      state: present
  register: address

- name: create a instance
  gcp_compute_instance:
      name: "test_object"
      machine_type: n1-standard-1
      disks:
      - auto_delete: true
        boot: true
        source: "{{ disk }}"
      metadata:
        startup-script-url: gs:://graphite-playground/bootstrap.sh
        cost-center: '12345'
      network_interfaces:
      - network: "{{ network }}"
        access_configs:
        - name: External NAT
          nat_ip: "{{ address }}"
          type: ONE_TO_ONE_NAT
      zone: us-central1-a
      project: "test_project"
      auth_kind: "service_account"
      service_account_file: "/tmp/auth.pem"
      state: present
'''

RETURN = '''
    can_ip_forward:
        description:
            - Allows this instance to send and receive packets with non-matching destination or
              source IPs. This is required if you plan to use this instance to forward routes.
        returned: success
        type: bool
    cpu_platform:
        description:
            - The CPU platform used by this instance.
        returned: success
        type: str
    creation_timestamp:
        description:
            - Creation timestamp in RFC3339 text format.
        returned: success
        type: str
    disks:
        description:
            - An array of disks that are associated with the instances that are created from this
              template.
        returned: success
        type: complex
        contains:
            auto_delete:
                description:
                    - Specifies whether the disk will be auto-deleted when the instance is deleted (but
                      not when the disk is detached from the instance).
                    - 'Tip: Disks should be set to autoDelete=true so that leftover disks are not left
                      behind on machine deletion.'
                returned: success
                type: bool
            boot:
                description:
                    - Indicates that this is a boot disk. The virtual machine will use the first partition
                      of the disk for its root filesystem.
                returned: success
                type: bool
            device_name:
                description:
                    - Specifies a unique device name of your choice that is reflected into the /dev/disk/by-id/google-*
                      tree of a Linux operating system running within the instance. This name can be used
                      to reference the device for mounting, resizing, and so on, from within the instance.
                returned: success
                type: str
            disk_encryption_key:
                description:
                    - Encrypts or decrypts a disk using a customer-supplied encryption key.
                returned: success
                type: complex
                contains:
                    raw_key:
                        description:
                            - Specifies a 256-bit customer-supplied encryption key, encoded in RFC 4648 base64
                              to either encrypt or decrypt this resource.
                        returned: success
                        type: str
                    rsa_encrypted_key:
                        description:
                            - Specifies an RFC 4648 base64 encoded, RSA-wrapped 2048-bit customer-supplied encryption
                              key to either encrypt or decrypt this resource.
                        returned: success
                        type: str
                    sha256:
                        description:
                            - The RFC 4648 base64 encoded SHA-256 hash of the customer-supplied encryption key
                              that protects this resource.
                        returned: success
                        type: str
            index:
                description:
                    - Assigns a zero-based index to this disk, where 0 is reserved for the boot disk.
                      For example, if you have many disks attached to an instance, each disk would have
                      a unique index number. If not specified, the server will choose an appropriate value.
                returned: success
                type: int
            initialize_params:
                description:
                    - Specifies the parameters for a new disk that will be created alongside the new instance.
                      Use initialization parameters to create boot disks or local SSDs attached to the
                      new instance.
                returned: success
                type: complex
                contains:
                    disk_name:
                        description:
                            - Specifies the disk name. If not specified, the default is to use the name of the
                              instance.
                        returned: success
                        type: str
                    disk_size_gb:
                        description:
                            - Specifies the size of the disk in base-2 GB.
                        returned: success
                        type: int
                    disk_type:
                        description:
                            - Reference to a gcompute_disk_type resource.
                            - Specifies the disk type to use to create the instance.
                            - If not specified, the default is pd-standard.
                        returned: success
                        type: str
                    source_image:
                        description:
                            - The source image to create this disk. When creating a new instance, one of initializeParams.sourceImage
                              or disks.source is required.  To create a disk with one of the public operating
                              system images, specify the image by its family name.
                        returned: success
                        type: str
                    source_image_encryption_key:
                        description:
                            - The customer-supplied encryption key of the source image. Required if the source
                              image is protected by a customer-supplied encryption key.
                            - Instance templates do not store customer-supplied encryption keys, so you cannot
                              create disks for instances in a managed instance group if the source images are
                              encrypted with your own keys.
                        returned: success
                        type: complex
                        contains:
                            raw_key:
                                description:
                                    - Specifies a 256-bit customer-supplied encryption key, encoded in RFC 4648 base64
                                      to either encrypt or decrypt this resource.
                                returned: success
                                type: str
                            sha256:
                                description:
                                    - The RFC 4648 base64 encoded SHA-256 hash of the customer-supplied encryption key
                                      that protects this resource.
                                returned: success
                                type: str
            interface:
                description:
                    - Specifies the disk interface to use for attaching this disk, which is either SCSI
                      or NVME. The default is SCSI.
                    - Persistent disks must always use SCSI and the request will fail if you attempt to
                      attach a persistent disk in any other format than SCSI.
                returned: success
                type: str
            mode:
                description:
                    - The mode in which to attach this disk, either READ_WRITE or READ_ONLY. If not specified,
                      the default is to attach the disk in READ_WRITE mode.
                returned: success
                type: str
            source:
                description:
                    - Reference to a gcompute_disk resource. When creating a new instance, one of initializeParams.sourceImage
                      or disks.source is required.
                    - If desired, you can also attach existing non-root persistent disks using this property.
                      This field is only applicable for persistent disks.
                returned: success
                type: dict
            type:
                description:
                    - Specifies the type of the disk, either SCRATCH or PERSISTENT. If not specified,
                      the default is PERSISTENT.
                returned: success
                type: str
    guest_accelerators:
        description:
            - List of the type and count of accelerator cards attached to the instance .
        returned: success
        type: complex
        contains:
            accelerator_count:
                description:
                    - The number of the guest accelerator cards exposed to this instance.
                returned: success
                type: int
            accelerator_type:
                description:
                    - Full or partial URL of the accelerator type resource to expose to this instance.
                returned: success
                type: str
    id:
        description:
            - The unique identifier for the resource. This identifier is defined by the server.
        returned: success
        type: int
    label_fingerprint:
        description:
            - A fingerprint for this request, which is essentially a hash of the metadata's contents
              and used for optimistic locking. The fingerprint is initially generated by Compute
              Engine and changes after every request to modify or update metadata. You must always
              provide an up-to-date fingerprint hash in order to update or change metadata.
        returned: success
        type: str
    metadata:
        description:
            - The metadata key/value pairs to assign to instances that are created from this template.
              These pairs can consist of custom metadata or predefined keys.
        returned: success
        type: dict
    machine_type:
        description:
            - A reference to a machine type which defines VM kind.
        returned: success
        type: str
    min_cpu_platform:
        description:
            - Specifies a minimum CPU platform for the VM instance. Applicable values are the
              friendly names of CPU platforms .
        returned: success
        type: str
    name:
        description:
            - The name of the resource, provided by the client when initially creating the resource.
              The resource name must be 1-63 characters long, and comply with RFC1035. Specifically,
              the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?`
              which means the first character must be a lowercase letter, and all following characters
              must be a dash, lowercase letter, or digit, except the last character, which cannot
              be a dash.
        returned: success
        type: str
    network_interfaces:
        description:
            - An array of configurations for this interface. This specifies how this interface
              is configured to interact with other network services, such as connecting to the
              internet. Only one network interface is supported per instance.
        returned: success
        type: complex
        contains:
            access_configs:
                description:
                    - An array of configurations for this interface. Currently, only one access config,
                      ONE_TO_ONE_NAT, is supported. If there are no accessConfigs specified, then this
                      instance will have no external internet access.
                returned: success
                type: complex
                contains:
                    name:
                        description:
                            - The name of this access configuration. The default and recommended name is External
                              NAT but you can use any arbitrary string you would like. For example, My external
                              IP or Network Access.
                        returned: success
                        type: str
                    nat_ip:
                        description:
                            - Specifies the title of a gcompute_address.
                            - An external IP address associated with this instance.
                            - Specify an unused static external IP address available to the project or leave this
                              field undefined to use an IP from a shared ephemeral IP address pool. If you specify
                              a static external IP address, it must live in the same region as the zone of the
                              instance.
                        returned: success
                        type: dict
                    type:
                        description:
                            - The type of configuration. The default and only option is ONE_TO_ONE_NAT.
                        returned: success
                        type: str
            alias_ip_ranges:
                description:
                    - An array of alias IP ranges for this network interface. Can only be specified for
                      network interfaces on subnet-mode networks.
                returned: success
                type: complex
                contains:
                    ip_cidr_range:
                        description:
                            - The IP CIDR range represented by this alias IP range.
                            - This IP CIDR range must belong to the specified subnetwork and cannot contain IP
                              addresses reserved by system or used by other network interfaces. This range may
                              be a single IP address (e.g. 10.2.3.4), a netmask (e.g. /24) or a CIDR format string
                              (e.g. 10.1.2.0/24).
                        returned: success
                        type: str
                    subnetwork_range_name:
                        description:
                            - Optional subnetwork secondary range name specifying the secondary range from which
                              to allocate the IP CIDR range for this alias IP range. If left unspecified, the
                              primary range of the subnetwork will be used.
                        returned: success
                        type: str
            name:
                description:
                    - The name of the network interface, generated by the server. For network devices,
                      these are eth0, eth1, etc .
                returned: success
                type: str
            network:
                description:
                    - Specifies the title of an existing gcompute_network.  When creating an instance,
                      if neither the network nor the subnetwork is specified, the default network global/networks/default
                      is used; if the network is not specified but the subnetwork is specified, the network
                      is inferred.
                returned: success
                type: dict
            network_ip:
                description:
                    - An IPv4 internal network address to assign to the instance for this network interface.
                      If not specified by the user, an unused internal IP is assigned by the system.
                returned: success
                type: str
            subnetwork:
                description:
                    - Reference to a gcompute_subnetwork resource.
                    - If the network resource is in legacy mode, do not provide this property.  If the
                      network is in auto subnet mode, providing the subnetwork is optional. If the network
                      is in custom subnet mode, then this field should be specified.
                returned: success
                type: dict
    scheduling:
        description:
            - Sets the scheduling options for this instance.
        returned: success
        type: complex
        contains:
            automatic_restart:
                description:
                    - Specifies whether the instance should be automatically restarted if it is terminated
                      by Compute Engine (not terminated by a user).
                    - You can only set the automatic restart option for standard instances. Preemptible
                      instances cannot be automatically restarted.
                returned: success
                type: bool
            on_host_maintenance:
                description:
                    - Defines the maintenance behavior for this instance. For standard instances, the
                      default behavior is MIGRATE. For preemptible instances, the default and only possible
                      behavior is TERMINATE.
                    - For more information, see Setting Instance Scheduling Options.
                returned: success
                type: str
            preemptible:
                description:
                    - Defines whether the instance is preemptible. This can only be set during instance
                      creation, it cannot be set or changed after the instance has been created.
                returned: success
                type: bool
    service_accounts:
        description:
            - A list of service accounts, with their specified scopes, authorized for this instance.
              Only one service account per VM instance is supported.
        returned: success
        type: complex
        contains:
            email:
                description:
                    - Email address of the service account.
                returned: success
                type: str
            scopes:
                description:
                    - The list of scopes to be made available for this service account.
                returned: success
                type: list
    status:
        description:
            - 'The status of the instance. One of the following values: PROVISIONING, STAGING,
              RUNNING, STOPPING, SUSPENDING, SUSPENDED, and TERMINATED.'
        returned: success
        type: str
    status_message:
        description:
            - An optional, human-readable explanation of the status.
        returned: success
        type: str
    tags:
        description:
            - A list of tags to apply to this instance. Tags are used to identify valid sources
              or targets for network firewalls and are specified by the client during instance
              creation. The tags can be later modified by the setTags method. Each tag within
              the list must comply with RFC1035.
        returned: success
        type: complex
        contains:
            fingerprint:
                description:
                    - Specifies a fingerprint for this request, which is essentially a hash of the metadata's
                      contents and used for optimistic locking.
                    - The fingerprint is initially generated by Compute Engine and changes after every
                      request to modify or update metadata. You must always provide an up-to-date fingerprint
                      hash in order to update or change metadata.
                returned: success
                type: str
            items:
                description:
                    - An array of tags. Each tag must be 1-63 characters long, and comply with RFC1035.
                returned: success
                type: list
    zone:
        description:
            - A reference to the zone where the machine resides.
        returned: success
        type: str
'''

################################################################################
# Imports
################################################################################

from ansible.module_utils.gcp_utils import navigate_hash, GcpSession, GcpModule, GcpRequest, remove_nones_from_dict, replace_resource_dict
import json
import re
import time

################################################################################
# Main
################################################################################


def main():
    """Main function"""

    module = GcpModule(
        argument_spec=dict(
            state=dict(default='present', choices=['present', 'absent'], type='str'),
            can_ip_forward=dict(type='bool'),
            disks=dict(type='list', elements='dict', options=dict(
                auto_delete=dict(type='bool'),
                boot=dict(type='bool'),
                device_name=dict(type='str'),
                disk_encryption_key=dict(type='dict', options=dict(
                    raw_key=dict(type='str'),
                    rsa_encrypted_key=dict(type='str'),
                    sha256=dict(type='str')
                )),
                index=dict(type='int'),
                initialize_params=dict(type='dict', options=dict(
                    disk_name=dict(type='str'),
                    disk_size_gb=dict(type='int'),
                    disk_type=dict(type='str'),
                    source_image=dict(type='str'),
                    source_image_encryption_key=dict(type='dict', options=dict(
                        raw_key=dict(type='str'),
                        sha256=dict(type='str')
                    ))
                )),
                interface=dict(type='str', choices=['SCSI', 'NVME']),
                mode=dict(type='str', choices=['READ_WRITE', 'READ_ONLY']),
                source=dict(type='dict'),
                type=dict(type='str', choices=['SCRATCH', 'PERSISTENT'])
            )),
            guest_accelerators=dict(type='list', elements='dict', options=dict(
                accelerator_count=dict(type='int'),
                accelerator_type=dict(type='str')
            )),
            label_fingerprint=dict(type='str'),
            metadata=dict(type='dict'),
            machine_type=dict(type='str'),
            min_cpu_platform=dict(type='str'),
            name=dict(type='str'),
            network_interfaces=dict(type='list', elements='dict', options=dict(
                access_configs=dict(type='list', elements='dict', options=dict(
                    name=dict(required=True, type='str'),
                    nat_ip=dict(type='dict'),
                    type=dict(required=True, type='str', choices=['ONE_TO_ONE_NAT'])
                )),
                alias_ip_ranges=dict(type='list', elements='dict', options=dict(
                    ip_cidr_range=dict(type='str'),
                    subnetwork_range_name=dict(type='str')
                )),
                name=dict(type='str'),
                network=dict(type='dict'),
                network_ip=dict(type='str'),
                subnetwork=dict(type='dict')
            )),
            scheduling=dict(type='dict', options=dict(
                automatic_restart=dict(type='bool'),
                on_host_maintenance=dict(type='str'),
                preemptible=dict(type='bool')
            )),
            service_accounts=dict(type='list', elements='dict', options=dict(
                email=dict(type='str'),
                scopes=dict(type='list', elements='str')
            )),
            tags=dict(type='dict', options=dict(
                fingerprint=dict(type='str'),
                items=dict(type='list', elements='str')
            )),
            zone=dict(required=True, type='str')
        )
    )

    if not module.params['scopes']:
        module.params['scopes'] = ['https://www.googleapis.com/auth/compute']

    state = module.params['state']
    kind = 'compute#instance'

    fetch = fetch_resource(module, self_link(module), kind)
    changed = False

    if fetch:
        if state == 'present':
            if is_different(module, fetch):
                fetch = update(module, self_link(module), kind)
                changed = True
        else:
            delete(module, self_link(module), kind)
            fetch = {}
            changed = True
    else:
        if state == 'present':
            fetch = create(module, collection(module), kind)
            changed = True
        else:
            fetch = {}

    fetch.update({'changed': changed})

    module.exit_json(**fetch)


def create(module, link, kind):
    auth = GcpSession(module, 'compute')
    return wait_for_operation(module, auth.post(link, resource_to_request(module)))


def update(module, link, kind):
    auth = GcpSession(module, 'compute')
    return wait_for_operation(module, auth.put(link, resource_to_request(module)))


def delete(module, link, kind):
    auth = GcpSession(module, 'compute')
    return wait_for_operation(module, auth.delete(link))


def resource_to_request(module):
    request = {
        u'kind': 'compute#instance',
        u'canIpForward': module.params.get('can_ip_forward'),
        u'disks': InstanceDisksArray(module.params.get('disks', []), module).to_request(),
        u'guestAccelerators': InstanceGuestAcceleratorsArray(module.params.get('guest_accelerators', []), module).to_request(),
        u'labelFingerprint': module.params.get('label_fingerprint'),
        u'metadata': module.params.get('metadata'),
        u'machineType': machine_type_selflink(module.params.get('machine_type'), module.params),
        u'minCpuPlatform': module.params.get('min_cpu_platform'),
        u'name': module.params.get('name'),
        u'networkInterfaces': InstanceNetworkInterfacesArray(module.params.get('network_interfaces', []), module).to_request(),
        u'scheduling': InstanceScheduling(module.params.get('scheduling', {}), module).to_request(),
        u'serviceAccounts': InstanceServiceAccountsArray(module.params.get('service_accounts', []), module).to_request(),
        u'tags': InstanceTags(module.params.get('tags', {}), module).to_request()
    }
    request = encode_request(request, module)
    return_vals = {}
    for k, v in request.items():
        if v:
            return_vals[k] = v

    return return_vals


def fetch_resource(module, link, kind):
    auth = GcpSession(module, 'compute')
    return return_if_object(module, auth.get(link), kind)


def self_link(module):
    return "https://www.googleapis.com/compute/v1/projects/{project}/zones/{zone}/instances/{name}".format(**module.params)


def collection(module):
    return "https://www.googleapis.com/compute/v1/projects/{project}/zones/{zone}/instances".format(**module.params)


def return_if_object(module, response, kind):
    # If not found, return nothing.
    if response.status_code == 404:
        return None

    # If no content, return nothing.
    if response.status_code == 204:
        return None

    try:
        module.raise_for_status(response)
        result = response.json()
    except getattr(json.decoder, 'JSONDecodeError', ValueError) as inst:
        module.fail_json(msg="Invalid JSON response with error: %s" % inst)

    result = decode_response(result, module)

    if navigate_hash(result, ['error', 'errors']):
        module.fail_json(msg=navigate_hash(result, ['error', 'errors']))
    if result['kind'] != kind:
        module.fail_json(msg="Incorrect result: {kind}".format(**result))

    return result


def is_different(module, response):
    request = resource_to_request(module)
    response = response_to_hash(module, response)
    request = decode_response(request, module)

    # Remove all output-only from response.
    response_vals = {}
    for k, v in response.items():
        if k in request:
            response_vals[k] = v

    request_vals = {}
    for k, v in request.items():
        if k in response:
            request_vals[k] = v

    return GcpRequest(request_vals) != GcpRequest(response_vals)


# Remove unnecessary properties from the response.
# This is for doing comparisons with Ansible's current parameters.
def response_to_hash(module, response):
    return {
        u'canIpForward': response.get(u'canIpForward'),
        u'cpuPlatform': response.get(u'cpuPlatform'),
        u'creationTimestamp': response.get(u'creationTimestamp'),
        u'disks': InstanceDisksArray(module.params.get('disks', []), module).to_request(),
        u'guestAccelerators': InstanceGuestAcceleratorsArray(response.get(u'guestAccelerators', []), module).from_response(),
        u'id': response.get(u'id'),
        u'labelFingerprint': response.get(u'labelFingerprint'),
        u'metadata': response.get(u'metadata'),
        u'machineType': response.get(u'machineType'),
        u'minCpuPlatform': response.get(u'minCpuPlatform'),
        u'name': response.get(u'name'),
        u'networkInterfaces': InstanceNetworkInterfacesArray(response.get(u'networkInterfaces', []), module).from_response(),
        u'scheduling': InstanceScheduling(response.get(u'scheduling', {}), module).from_response(),
        u'serviceAccounts': InstanceServiceAccountsArray(response.get(u'serviceAccounts', []), module).from_response(),
        u'status': response.get(u'status'),
        u'statusMessage': response.get(u'statusMessage'),
        u'tags': InstanceTags(response.get(u'tags', {}), module).from_response()
    }


def disk_type_selflink(name, params):
    if name is None:
        return
    url = r"https://www.googleapis.com/compute/v1/projects/.*/zones/{zone}/diskTypes/[a-z1-9\-]*"
    if not re.match(url, name):
        name = "https://www.googleapis.com/compute/v1/projects/{project}/zones/{zone}/diskTypes/%s".format(**params) % name
    return name


def machine_type_selflink(name, params):
    if name is None:
        return
    url = r"https://www.googleapis.com/compute/v1/projects/.*/zones/{zone}/machineTypes/[a-z1-9\-]*"
    if not re.match(url, name):
        name = "https://www.googleapis.com/compute/v1/projects/{project}/zones/{zone}/machineTypes/%s".format(**params) % name
    return name


def async_op_url(module, extra_data=None):
    if extra_data is None:
        extra_data = {}
    url = "https://www.googleapis.com/compute/v1/projects/{project}/zones/{zone}/operations/{op_id}"
    combined = extra_data.copy()
    combined.update(module.params)
    return url.format(**combined)


def wait_for_operation(module, response):
    op_result = return_if_object(module, response, 'compute#operation')
    if op_result is None:
        return {}
    status = navigate_hash(op_result, ['status'])
    wait_done = wait_for_completion(status, op_result, module)
    return fetch_resource(module, navigate_hash(wait_done, ['targetLink']), 'compute#instance')


def wait_for_completion(status, op_result, module):
    op_id = navigate_hash(op_result, ['name'])
    op_uri = async_op_url(module, {'op_id': op_id})
    while status != 'DONE':
        raise_if_errors(op_result, ['error', 'errors'], 'message')
        time.sleep(1.0)
        if status not in ['PENDING', 'RUNNING', 'DONE']:
            module.fail_json(msg="Invalid result %s" % status)
        op_result = fetch_resource(module, op_uri, 'compute#operation')
        status = navigate_hash(op_result, ['status'])
    return op_result


def raise_if_errors(response, err_path, module):
    errors = navigate_hash(response, err_path)
    if errors is not None:
        module.fail_json(msg=errors)


def encode_request(request, module):
    if 'metadata' in request and request['metadata'] is not None:
        request['metadata'] = metadata_encoder(request['metadata'])
    return request


def decode_response(response, module):
    if 'metadata' in response:
        response['metadata'] = metadata_decoder(response['metadata'])
    return response


# TODO(alexstephen): Implement updating metadata on exsiting resources.

# Expose instance 'metadata' as a simple name/value pair hash. However the API
# defines metadata as a NestedObject with the following layout:
#
# metadata {
#   fingerprint: 'hash-of-last-metadata'
#   items: [
#     {
#       key: 'metadata1-key'
#       value: 'metadata1-value'
#     },
#     ...
#   ]
# }
#
def metadata_encoder(metadata):
    metadata_new = []
    for key in metadata:
        value = metadata[key]
        metadata_new.append({
            "key": key,
            "value": value
        })
    return {
        'items': metadata_new
    }


# Map metadata.items[]{key:,value:} => metadata[key]=value
def metadata_decoder(metadata):
    items = {}
    if 'items' in metadata:
        metadata_items = metadata['items']
        for item in metadata_items:
            items[item['key']] = item['value']
    return items


class InstanceDisksArray(object):
    def __init__(self, request, module):
        self.module = module
        if request:
            self.request = request
        else:
            self.request = []

    def to_request(self):
        items = []
        for item in self.request:
            items.append(self._request_for_item(item))
        return items

    def from_response(self):
        items = []
        for item in self.request:
            items.append(self._response_from_item(item))
        return items

    def _request_for_item(self, item):
        return remove_nones_from_dict({
            u'autoDelete': item.get('auto_delete'),
            u'boot': item.get('boot'),
            u'deviceName': item.get('device_name'),
            u'diskEncryptionKey': InstanceDiskEncryptionKey(item.get('disk_encryption_key', {}), self.module).to_request(),
            u'index': item.get('index'),
            u'initializeParams': InstanceInitializeParams(item.get('initialize_params', {}), self.module).to_request(),
            u'interface': item.get('interface'),
            u'mode': item.get('mode'),
            u'source': replace_resource_dict(item.get(u'source', {}), 'selfLink'),
            u'type': item.get('type')
        })

    def _response_from_item(self, item):
        return remove_nones_from_dict({
            u'autoDelete': item.get(u'autoDelete'),
            u'boot': item.get(u'boot'),
            u'deviceName': item.get(u'deviceName'),
            u'diskEncryptionKey': InstanceDiskEncryptionKey(item.get(u'diskEncryptionKey', {}), self.module).from_response(),
            u'index': item.get(u'index'),
            u'initializeParams': InstanceInitializeParams(self.module.params.get('initialize_params', {}), self.module).to_request(),
            u'interface': item.get(u'interface'),
            u'mode': item.get(u'mode'),
            u'source': item.get(u'source'),
            u'type': item.get(u'type')
        })


class InstanceDiskEncryptionKey(object):
    def __init__(self, request, module):
        self.module = module
        if request:
            self.request = request
        else:
            self.request = {}

    def to_request(self):
        return remove_nones_from_dict({
            u'rawKey': self.request.get('raw_key'),
            u'rsaEncryptedKey': self.request.get('rsa_encrypted_key'),
            u'sha256': self.request.get('sha256')
        })

    def from_response(self):
        return remove_nones_from_dict({
            u'rawKey': self.request.get(u'rawKey'),
            u'rsaEncryptedKey': self.request.get(u'rsaEncryptedKey'),
            u'sha256': self.request.get(u'sha256')
        })


class InstanceInitializeParams(object):
    def __init__(self, request, module):
        self.module = module
        if request:
            self.request = request
        else:
            self.request = {}

    def to_request(self):
        return remove_nones_from_dict({
            u'diskName': self.request.get('disk_name'),
            u'diskSizeGb': self.request.get('disk_size_gb'),
            u'diskType': disk_type_selflink(self.request.get('disk_type'), self.module.params),
            u'sourceImage': self.request.get('source_image'),
            u'sourceImageEncryptionKey': InstanceSourceImageEncryptionKey(self.request.get('source_image_encryption_key', {}), self.module).to_request()
        })

    def from_response(self):
        return remove_nones_from_dict({
            u'diskName': self.request.get(u'diskName'),
            u'diskSizeGb': self.request.get(u'diskSizeGb'),
            u'diskType': self.request.get(u'diskType'),
            u'sourceImage': self.request.get(u'sourceImage'),
            u'sourceImageEncryptionKey': InstanceSourceImageEncryptionKey(self.request.get(u'sourceImageEncryptionKey', {}), self.module).from_response()
        })


class InstanceSourceImageEncryptionKey(object):
    def __init__(self, request, module):
        self.module = module
        if request:
            self.request = request
        else:
            self.request = {}

    def to_request(self):
        return remove_nones_from_dict({
            u'rawKey': self.request.get('raw_key'),
            u'sha256': self.request.get('sha256')
        })

    def from_response(self):
        return remove_nones_from_dict({
            u'rawKey': self.request.get(u'rawKey'),
            u'sha256': self.request.get(u'sha256')
        })


class InstanceGuestAcceleratorsArray(object):
    def __init__(self, request, module):
        self.module = module
        if request:
            self.request = request
        else:
            self.request = []

    def to_request(self):
        items = []
        for item in self.request:
            items.append(self._request_for_item(item))
        return items

    def from_response(self):
        items = []
        for item in self.request:
            items.append(self._response_from_item(item))
        return items

    def _request_for_item(self, item):
        return remove_nones_from_dict({
            u'acceleratorCount': item.get('accelerator_count'),
            u'acceleratorType': item.get('accelerator_type')
        })

    def _response_from_item(self, item):
        return remove_nones_from_dict({
            u'acceleratorCount': item.get(u'acceleratorCount'),
            u'acceleratorType': item.get(u'acceleratorType')
        })


class InstanceNetworkInterfacesArray(object):
    def __init__(self, request, module):
        self.module = module
        if request:
            self.request = request
        else:
            self.request = []

    def to_request(self):
        items = []
        for item in self.request:
            items.append(self._request_for_item(item))
        return items

    def from_response(self):
        items = []
        for item in self.request:
            items.append(self._response_from_item(item))
        return items

    def _request_for_item(self, item):
        return remove_nones_from_dict({
            u'accessConfigs': InstanceAccessConfigsArray(item.get('access_configs', []), self.module).to_request(),
            u'aliasIpRanges': InstanceAliasIpRangesArray(item.get('alias_ip_ranges', []), self.module).to_request(),
            u'name': item.get('name'),
            u'network': replace_resource_dict(item.get(u'network', {}), 'selfLink'),
            u'networkIP': item.get('network_ip'),
            u'subnetwork': replace_resource_dict(item.get(u'subnetwork', {}), 'selfLink')
        })

    def _response_from_item(self, item):
        return remove_nones_from_dict({
            u'accessConfigs': InstanceAccessConfigsArray(item.get(u'accessConfigs', []), self.module).from_response(),
            u'aliasIpRanges': InstanceAliasIpRangesArray(item.get(u'aliasIpRanges', []), self.module).from_response(),
            u'name': item.get(u'name'),
            u'network': item.get(u'network'),
            u'networkIP': item.get(u'networkIP'),
            u'subnetwork': item.get(u'subnetwork')
        })


class InstanceAccessConfigsArray(object):
    def __init__(self, request, module):
        self.module = module
        if request:
            self.request = request
        else:
            self.request = []

    def to_request(self):
        items = []
        for item in self.request:
            items.append(self._request_for_item(item))
        return items

    def from_response(self):
        items = []
        for item in self.request:
            items.append(self._response_from_item(item))
        return items

    def _request_for_item(self, item):
        return remove_nones_from_dict({
            u'name': item.get('name'),
            u'natIP': replace_resource_dict(item.get(u'nat_ip', {}), 'address'),
            u'type': item.get('type')
        })

    def _response_from_item(self, item):
        return remove_nones_from_dict({
            u'name': item.get(u'name'),
            u'natIP': item.get(u'natIP'),
            u'type': item.get(u'type')
        })


class InstanceAliasIpRangesArray(object):
    def __init__(self, request, module):
        self.module = module
        if request:
            self.request = request
        else:
            self.request = []

    def to_request(self):
        items = []
        for item in self.request:
            items.append(self._request_for_item(item))
        return items

    def from_response(self):
        items = []
        for item in self.request:
            items.append(self._response_from_item(item))
        return items

    def _request_for_item(self, item):
        return remove_nones_from_dict({
            u'ipCidrRange': item.get('ip_cidr_range'),
            u'subnetworkRangeName': item.get('subnetwork_range_name')
        })

    def _response_from_item(self, item):
        return remove_nones_from_dict({
            u'ipCidrRange': item.get(u'ipCidrRange'),
            u'subnetworkRangeName': item.get(u'subnetworkRangeName')
        })


class InstanceScheduling(object):
    def __init__(self, request, module):
        self.module = module
        if request:
            self.request = request
        else:
            self.request = {}

    def to_request(self):
        return remove_nones_from_dict({
            u'automaticRestart': self.request.get('automatic_restart'),
            u'onHostMaintenance': self.request.get('on_host_maintenance'),
            u'preemptible': self.request.get('preemptible')
        })

    def from_response(self):
        return remove_nones_from_dict({
            u'automaticRestart': self.request.get(u'automaticRestart'),
            u'onHostMaintenance': self.request.get(u'onHostMaintenance'),
            u'preemptible': self.request.get(u'preemptible')
        })


class InstanceServiceAccountsArray(object):
    def __init__(self, request, module):
        self.module = module
        if request:
            self.request = request
        else:
            self.request = []

    def to_request(self):
        items = []
        for item in self.request:
            items.append(self._request_for_item(item))
        return items

    def from_response(self):
        items = []
        for item in self.request:
            items.append(self._response_from_item(item))
        return items

    def _request_for_item(self, item):
        return remove_nones_from_dict({
            u'email': item.get('email'),
            u'scopes': item.get('scopes')
        })

    def _response_from_item(self, item):
        return remove_nones_from_dict({
            u'email': item.get(u'email'),
            u'scopes': item.get(u'scopes')
        })


class InstanceTags(object):
    def __init__(self, request, module):
        self.module = module
        if request:
            self.request = request
        else:
            self.request = {}

    def to_request(self):
        return remove_nones_from_dict({
            u'fingerprint': self.request.get('fingerprint'),
            u'items': self.request.get('items')
        })

    def from_response(self):
        return remove_nones_from_dict({
            u'fingerprint': self.request.get(u'fingerprint'),
            u'items': self.request.get(u'items')
        })


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# (c) 2017 Apstra Inc, <community@apstra.com>
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['deprecated'],
                    'supported_by': 'community'}


DOCUMENTATION = '''
---
module: aos_login
author: jeremy@apstra.com (@jeremyschulman)
version_added: "2.3"
short_description: Login to AOS server for session token
deprecated:
    removed_in: "2.9"
    why: This module does not support AOS 2.1 or later
    alternative: See new modules at U(https://www.ansible.com/ansible-apstra).
description:
  - Obtain the AOS server session token by providing the required
    username and password credentials.  Upon successful authentication,
    this module will return the session-token that is required by all
    subsequent AOS module usage. On success the module will automatically populate
    ansible facts with the variable I(aos_session)
    This module is not idempotent and do not support check mode.
requirements:
  - "aos-pyez >= 0.6.1"
options:
  server:
    description:
      - Address of the AOS Server on which you want to open a connection.
    required: true
  port:
    description:
      - Port number to use when connecting to the AOS server.
    default: 443
  user:
    description:
      - Login username to use when connecting to the AOS server.
    default: admin
  passwd:
    description:
      - Password to use when connecting to the AOS server.
    default: admin
'''

EXAMPLES = '''

- name: Create a session with the AOS-server
  aos_login:
    server: "{{ inventory_hostname }}"
    user: admin
    passwd: admin

- name: Use the newly created session (register is not mandatory)
  aos_ip_pool:
    session: "{{ aos_session }}"
    name: my_ip_pool
    state: present
'''

RETURNS = '''
aos_session:
  description: Authenticated session information
  returned: always
  type: dict
  sample: { 'url': <str>, 'headers': {...} }
'''

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.network.aos.aos import check_aos_version

try:
    from apstra.aosom.session import Session
    import apstra.aosom.exc as aosExc

    HAS_AOS_PYEZ = True
except ImportError:
    HAS_AOS_PYEZ = False


def aos_login(module):

    mod_args = module.params

    aos = Session(server=mod_args['server'], port=mod_args['port'],
                  user=mod_args['user'], passwd=mod_args['passwd'])

    try:
        aos.login()
    except aosExc.LoginServerUnreachableError:
        module.fail_json(
            msg="AOS-server [%s] API not available/reachable, check server" % aos.server)

    except aosExc.LoginAuthError:
        module.fail_json(msg="AOS-server login credentials failed")

    module.exit_json(changed=False,
                     ansible_facts=dict(aos_session=aos.session),
                     aos_session=dict(aos_session=aos.session))


def main():
    module = AnsibleModule(
        argument_spec=dict(
            server=dict(required=True),
            port=dict(default='443', type="int"),
            user=dict(default='admin'),
            passwd=dict(default='admin', no_log=True)))

    if not HAS_AOS_PYEZ:
        module.fail_json(msg='aos-pyez is not installed.  Please see details '
                             'here: https://github.com/Apstra/aos-pyez')

    # Check if aos-pyez is present and match the minimum version
    check_aos_version(module, '0.6.1')

    aos_login(module)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2017, Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

# this is a windows documentation stub.  actual code lives in the .ps1
# file of the same name

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: win_whoami
version_added: "2.5"
short_description: Get information about the current user and process
description:
- Designed to return the same information as the C(whoami /all) command.
- Also includes information missing from C(whoami) such as logon metadata like
  logon rights, id, type.
notes:
- If running this module with a non admin user, the logon rights will be an
  empty list as Administrator rights are required to query LSA for the
  information.
author:
- Jordan Borean (@jborean93)
'''

EXAMPLES = r'''
- name: get whoami information
  win_whoami:
'''

RETURN = r'''
authentication_package:
  description: The name of the authentication package used to authenticate the
    user in the session.
  returned: success
  type: str
  sample: Negotiate
user_flags:
  description: The user flags for the logon session, see UserFlags in
    U(https://msdn.microsoft.com/en-us/library/windows/desktop/aa380128).
  returned: success
  type: str
  sample: Winlogon
upn:
  description: The user principal name of the current user.
  returned: success
  type: str
  sample: Administrator@DOMAIN.COM
logon_type:
  description: The logon type that identifies the logon method, see
    U(https://msdn.microsoft.com/en-us/library/windows/desktop/aa380129.aspx).
  returned: success
  type: str
  sample: Network
privileges:
  description: A dictionary of privileges and their state on the logon token.
  returned: success
  type: dict
  sample: {
      "SeChangeNotifyPrivileges": "enabled-by-default",
      "SeRemoteShutdownPrivilege": "disabled",
      "SeDebugPrivilege": "enabled"
  }
label:
  description: The mandatory label set to the logon session.
  returned: success
  type: complex
  contains:
    domain_name:
      description: The domain name of the label SID.
      returned: success
      type: str
      sample: Mandatory Label
    sid:
      description: The SID in string form.
      returned: success
      type: str
      sample: S-1-16-12288
    account_name:
      description: The account name of the label SID.
      returned: success
      type: str
      sample: High Mandatory Level
    type:
      description: The type of SID.
      returned: success
      type: str
      sample: Label
impersonation_level:
  description: The impersonation level of the token, only valid if
    C(token_type) is C(TokenImpersonation), see
    U(https://msdn.microsoft.com/en-us/library/windows/desktop/aa379572.aspx).
  returned: success
  type: str
  sample: SecurityAnonymous
login_time:
  description: The logon time in ISO 8601 format
  returned: success
  type: str
  sample: '2017-11-27T06:24:14.3321665+10:00'
groups:
  description: A list of groups and attributes that the user is a member of.
  returned: success
  type: list
  sample: [
      {
          "account_name": "Domain Users",
          "domain_name": "DOMAIN",
          "attributes": [
              "Mandatory",
              "Enabled by default",
              "Enabled"
          ],
          "sid": "S-1-5-21-1654078763-769949647-2968445802-513",
          "type": "Group"
      },
      {
          "account_name": "Administrators",
          "domain_name": "BUILTIN",
          "attributes": [
              "Mandatory",
              "Enabled by default",
              "Enabled",
              "Owner"
          ],
          "sid": "S-1-5-32-544",
          "type": "Alias"
      }
  ]
account:
  description: The running account SID details.
  returned: success
  type: complex
  contains:
    domain_name:
      description: The domain name of the account SID.
      returned: success
      type: str
      sample: DOMAIN
    sid:
      description: The SID in string form.
      returned: success
      type: str
      sample: S-1-5-21-1654078763-769949647-2968445802-500
    account_name:
      description: The account name of the account SID.
      returned: success
      type: str
      sample: Administrator
    type:
      description: The type of SID.
      returned: success
      type: str
      sample: User
login_domain:
  description: The name of the domain used to authenticate the owner of the
    session.
  returned: success
  type: str
  sample: DOMAIN
rights:
  description: A list of logon rights assigned to the logon.
  returned: success and running user is a member of the local Administrators group
  type: list
  sample: [
      "SeNetworkLogonRight",
      "SeInteractiveLogonRight",
      "SeBatchLogonRight",
      "SeRemoteInteractiveLogonRight"
  ]
logon_server:
  description: The name of the server used to authentcate the owner of the
    logon session.
  returned: success
  type: str
  sample: DC01
logon_id:
  description: The unique identifier of the logon session.
  returned: success
  type: int
  sample: 20470143
dns_domain_name:
  description: The DNS name of the logon session, this is an empty string if
    this is not set.
  returned: success
  type: str
  sample: DOMAIN.COM
token_type:
  description: The token type to indicate whether it is a primary or
    impersonation token.
  returned: success
  type: str
  sample: TokenPrimary
'''



#!/usr/bin/python
# -*- coding: utf-8 -*-

#  Copyright (c) 2017 Citrix Systems
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}


DOCUMENTATION = '''
---
module: netscaler_cs_policy
short_description: Manage content switching policy
description:
    - Manage content switching policy.
    - "This module is intended to run either on the ansible  control node or a bastion (jumpserver) with access to the actual netscaler instance."

version_added: "2.4"

author: George Nikolopoulos (@giorgos-nikolopoulos)

options:

    policyname:
        description:
            - >-
                Name for the content switching policy. Must begin with an ASCII alphanumeric or underscore C(_)
                character, and must contain only ASCII alphanumeric, underscore, hash C(#), period C(.), space C( ), colon
                C(:), at sign C(@), equal sign C(=), and hyphen C(-) characters. Cannot be changed after a policy is
                created.
            - "The following requirement applies only to the NetScaler CLI:"
            - >-
                If the name includes one or more spaces, enclose the name in double or single quotation marks (for
                example, my policy or my policy).
            - "Minimum length = 1"

    url:
        description:
            - >-
                URL string that is matched with the URL of a request. Can contain a wildcard character. Specify the
                string value in the following format: C([[prefix] [*]] [.suffix]).
            - "Minimum length = 1"
            - "Maximum length = 208"

    rule:
        description:
            - >-
                Expression, or name of a named expression, against which traffic is evaluated. Written in the classic
                or default syntax.
            - "Note:"
            - >-
                Maximum length of a string literal in the expression is 255 characters. A longer string can be split
                into smaller strings of up to 255 characters each, and the smaller strings concatenated with the +
                operator. For example, you can create a 500-character string as follows: '"<string of 255
                characters>" + "<string of 245 characters>"'

    domain:
        description:
            - "The domain name. The string value can range to 63 characters."
            - "Minimum length = 1"

    action:
        description:
            - >-
                Content switching action that names the target load balancing virtual server to which the traffic is
                switched.

extends_documentation_fragment: netscaler
requirements:
    - nitro python sdk
'''

EXAMPLES = '''
- name: Create url cs policy
  delegate_to: localhost
  netscaler_cs_policy:
    nsip: 172.18.0.2
    nitro_user: nsroot
    nitro_pass: nsroot
    validate_certs: no

    state: present

    policyname: policy_1
    url: /example/
'''

RETURN = '''
loglines:
    description: list of logged messages by the module
    returned: always
    type: list
    sample: ['message 1', 'message 2']

msg:
    description: Message detailing the failure reason
    returned: failure
    type: str
    sample: "Could not load nitro python sdk"

diff:
    description: List of differences between the actual configured object and the configuration specified in the module
    returned: failure
    type: dict
    sample: { 'url': 'difference. ours: (str) example1 other: (str) /example1' }
'''

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.network.netscaler.netscaler import ConfigProxy, get_nitro_client, netscaler_common_arguments, log, loglines, ensure_feature_is_enabled
try:
    from nssrc.com.citrix.netscaler.nitro.resource.config.cs.cspolicy import cspolicy
    from nssrc.com.citrix.netscaler.nitro.exception.nitro_exception import nitro_exception
    PYTHON_SDK_IMPORTED = True
except ImportError as e:
    PYTHON_SDK_IMPORTED = False


def policy_exists(client, module):
    log('Checking if policy exists')
    if cspolicy.count_filtered(client, 'policyname:%s' % module.params['policyname']) > 0:
        return True
    else:
        return False


def policy_identical(client, module, cspolicy_proxy):
    log('Checking if defined policy is identical to configured')
    if cspolicy.count_filtered(client, 'policyname:%s' % module.params['policyname']) == 0:
        return False
    policy_list = cspolicy.get_filtered(client, 'policyname:%s' % module.params['policyname'])
    diff_dict = cspolicy_proxy.diff_object(policy_list[0])
    if 'ip' in diff_dict:
        del diff_dict['ip']
    if len(diff_dict) == 0:
        return True
    else:
        return False


def diff_list(client, module, cspolicy_proxy):
    policy_list = cspolicy.get_filtered(client, 'policyname:%s' % module.params['policyname'])
    return cspolicy_proxy.diff_object(policy_list[0])


def main():

    module_specific_arguments = dict(
        policyname=dict(type='str'),
        url=dict(type='str'),
        rule=dict(type='str'),
        domain=dict(type='str'),
        action=dict(type='str'),
    )

    hand_inserted_arguments = dict(
    )

    argument_spec = dict()

    argument_spec.update(netscaler_common_arguments)
    argument_spec.update(module_specific_arguments)
    argument_spec.update(hand_inserted_arguments)

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )
    module_result = dict(
        changed=False,
        failed=False,
        loglines=loglines,
    )

    # Fail the module if imports failed
    if not PYTHON_SDK_IMPORTED:
        module.fail_json(msg='Could not load nitro python sdk')

    # Fallthrough to rest of execution
    client = get_nitro_client(module)

    try:
        client.login()
    except nitro_exception as e:
        msg = "nitro exception during login. errorcode=%s, message=%s" % (str(e.errorcode), e.message)
        module.fail_json(msg=msg)
    except Exception as e:
        if str(type(e)) == "<class 'requests.exceptions.ConnectionError'>":
            module.fail_json(msg='Connection error %s' % str(e))
        elif str(type(e)) == "<class 'requests.exceptions.SSLError'>":
            module.fail_json(msg='SSL Error %s' % str(e))
        else:
            module.fail_json(msg='Unexpected error during login %s' % str(e))

    readwrite_attrs = [
        'policyname',
        'url',
        'rule',
        'domain',
        'action',
    ]
    readonly_attrs = [
        'vstype',
        'hits',
        'bindhits',
        'labelname',
        'labeltype',
        'priority',
        'activepolicy',
        'cspolicytype',
    ]

    transforms = {
    }

    # Instantiate config proxy
    cspolicy_proxy = ConfigProxy(
        actual=cspolicy(),
        client=client,
        attribute_values_dict=module.params,
        readwrite_attrs=readwrite_attrs,
        readonly_attrs=readonly_attrs,
        transforms=transforms,
    )

    try:
        ensure_feature_is_enabled(client, 'CS')

        # Apply appropriate state
        if module.params['state'] == 'present':
            log('Sanity checks for state present')
            if not policy_exists(client, module):
                if not module.check_mode:
                    cspolicy_proxy.add()
                    if module.params['save_config']:
                        client.save_config()
                module_result['changed'] = True
            elif not policy_identical(client, module, cspolicy_proxy):
                if not module.check_mode:
                    cspolicy_proxy.update()
                    if module.params['save_config']:
                        client.save_config()
                module_result['changed'] = True
            else:
                module_result['changed'] = False

            # Sanity check for state
            if not module.check_mode:
                log('Sanity checks for state present')
                if not policy_exists(client, module):
                    module.fail_json(msg='Policy does not exist', **module_result)
                if not policy_identical(client, module, cspolicy_proxy):
                    module.fail_json(msg='Policy differs from configured', diff=diff_list(client, module, cspolicy_proxy), **module_result)

        elif module.params['state'] == 'absent':
            log('Applying actions for state absent')
            if policy_exists(client, module):
                if not module.check_mode:
                    cspolicy_proxy.delete()
                    if module.params['save_config']:
                        client.save_config()
                module_result['changed'] = True
            else:
                module_result['changed'] = False

            # Sanity check for state
            if not module.check_mode:
                log('Sanity checks for state absent')
                if policy_exists(client, module):
                    module.fail_json(msg='Policy still exists', **module_result)

    except nitro_exception as e:
        msg = "nitro exception errorcode=%s, message=%s" % (str(e.errorcode), e.message)
        module.fail_json(msg=msg, **module_result)

    client.logout()
    module.exit_json(**module_result)


if __name__ == "__main__":
    main()



#
# (c) 2016 Red Hat Inc.
# (c) 2017 Paul Neumann
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

# Make coding more python3-ish
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import json

from ansible.compat.tests.mock import patch
from ansible.modules.network.ios import ios_logging
from units.modules.utils import set_module_args
from .ios_module import TestIosModule, load_fixture


class TestIosLoggingModule(TestIosModule):

    module = ios_logging

    def setUp(self):
        super(TestIosLoggingModule, self).setUp()

        self.mock_get_config = patch('ansible.modules.network.ios.ios_logging.get_config')
        self.get_config = self.mock_get_config.start()

        self.mock_load_config = patch('ansible.modules.network.ios.ios_logging.load_config')
        self.load_config = self.mock_load_config.start()

        self.mock_get_capabilities = patch('ansible.modules.network.ios.ios_logging.get_capabilities')
        self.get_capabilities = self.mock_get_capabilities.start()
        self.get_capabilities.return_value = {'device_info': {'network_os_version': '15.6(2)T'}}

    def tearDown(self):
        super(TestIosLoggingModule, self).tearDown()

        self.mock_get_config.stop()
        self.mock_load_config.stop()
        self.mock_get_capabilities.stop()

    def load_fixtures(self, commands=None):
        self.get_config.return_value = load_fixture('ios_logging_config.cfg')
        self.load_config.return_value = None

    def test_ios_logging_buffer_size_changed_implicit(self):
        set_module_args(dict(dest='buffered'))
        commands = ['logging buffered 4096']
        self.execute_module(changed=True, commands=commands)

    def test_ios_logging_buffer_size_changed_explicit(self):
        set_module_args(dict(dest='buffered', size=6000))
        commands = ['logging buffered 6000']
        self.execute_module(changed=True, commands=commands)



# Copyright (c), Sviatoslav Sydorenko <ssydoren@redhat.com> 2018
# Simplified BSD License (see licenses/simplified_bsd.txt or https://opensource.org/licenses/BSD-2-Clause)
"""Collection of low-level utility functions."""

from __future__ import absolute_import, division, print_function
__metaclass__ = type


from ansible.module_utils.six import binary_type, text_type
from ansible.module_utils.common._collections_compat import Sequence


def is_string(seq):
    """Identify whether the input has a string-like type (inclding bytes)."""
    return isinstance(seq, (text_type, binary_type))


def is_iterable(seq, include_strings=False):
    """Identify whether the input is an iterable."""
    if not include_strings and is_string(seq):
        return False

    try:
        iter(seq)
        return True
    except TypeError:
        return False


def is_sequence(seq, include_strings=False):
    """Identify whether the input is a sequence.

    Strings and bytes are not sequences here,
    unless ``include_string`` is ``True``.

    Non-indexable things are never of a sequence type.
    """
    if not include_strings and is_string(seq):
        return False

    return isinstance(seq, Sequence)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# (c) 2017, Ansible by Red Hat, inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'network'}


DOCUMENTATION = """
---
module: net_logging
version_added: "2.4"
author: "Ganesh Nalawade (@ganeshrn)"
short_description: Manage logging on network devices
description:
  - This module provides declarative management of logging
    on network devices.
options:
  dest:
    description:
      - Destination of the logs.
    choices: ['console', 'host']
  name:
    description:
      - If value of C(dest) is I(host) it indicates file-name
        the host name to be notified.
  facility:
    description:
      - Set logging facility.
  level:
    description:
      - Set logging severity levels.
  aggregate:
    description: List of logging definitions.
  purge:
    description:
      - Purge logging not defined in the I(aggregate) parameter.
    default: no
  state:
    description:
      - State of the logging configuration.
    default: present
    choices: ['present', 'absent']
"""

EXAMPLES = """
- name: configure console logging
  net_logging:
    dest: console
    facility: any
    level: critical

- name: remove console logging configuration
  net_logging:
    dest: console
    state: absent

- name: configure host logging
  net_logging:
    dest: host
    name: 192.0.2.1
    facility: kernel
    level: critical

- name: Configure file logging using aggregate
  net_logging:
    dest: file
    aggregate:
    - name: test-1
      facility: pfe
      level: critical
    - name: test-2
      facility: kernel
      level: emergency
- name: Delete file logging using aggregate
  net_logging:
    dest: file
    aggregate:
    - name: test-1
      facility: pfe
      level: critical
    - name: test-2
      facility: kernel
      level: emergency
    state: absent
"""

RETURN = """
commands:
  description: The list of configuration mode commands to send to the device
  returned: always, except for the platforms that use Netconf transport to manage the device.
  type: list
  sample:
    - logging console critical
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2015, Joseph Callen <jcallen () csc.com>
# Copyright: (c) 2017-2018, Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {
    'metadata_version': '1.1',
    'status': ['preview'],
    'supported_by': 'community'
}

DOCUMENTATION = '''
---
module: vmware_dvs_portgroup
short_description: Create or remove a Distributed vSwitch portgroup.
description:
    - Create or remove a Distributed vSwitch portgroup.
version_added: 2.0
author:
    - Joseph Callen (@jcpowermac)
    - Philippe Dellaert (@pdellaert) <philippe@dellaert.org>
notes:
    - Tested on vSphere 5.5
    - Tested on vSphere 6.5
requirements:
    - "python >= 2.6"
    - PyVmomi
options:
    portgroup_name:
        description:
            - The name of the portgroup that is to be created or deleted.
        required: True
    switch_name:
        description:
            - The name of the distributed vSwitch the port group should be created on.
        required: True
    vlan_id:
        description:
            - The VLAN ID that should be configured with the portgroup, use 0 for no VLAN.
            - 'If C(vlan_trunk) is configured to be I(true), this can be a range, example: 1-4094.'
        required: True
    num_ports:
        description:
            - The number of ports the portgroup should contain.
        required: True
    portgroup_type:
        description:
            - See VMware KB 1022312 regarding portgroup types.
        required: True
        choices:
            - 'earlyBinding'
            - 'lateBinding'
            - 'ephemeral'
    state:
        description:
            - Determines if the portgroup should be present or not.
        required: True
        type: bool
        choices:
            - 'present'
            - 'absent'
        version_added: '2.5'
    vlan_trunk:
        description:
            - Indicates whether this is a VLAN trunk or not.
        required: False
        default: False
        type: bool
        version_added: '2.5'
    network_policy:
        description:
            - Dictionary which configures the different security values for portgroup.
            - 'Valid attributes are:'
            - '- C(promiscuous) (bool): indicates whether promiscuous mode is allowed. (default: false)'
            - '- C(forged_transmits) (bool): indicates whether forged transmits are allowed. (default: false)'
            - '- C(mac_changes) (bool): indicates whether mac changes are allowed. (default: false)'
        required: False
        version_added: '2.5'
        default: {
            promiscuous: False,
            forged_transmits: False,
            mac_changes: False,
        }
    teaming_policy:
        description:
            - Dictionary which configures the different teaming values for portgroup.
            - 'Valid attributes are:'
            - '- C(load_balance_policy) (string): Network adapter teaming policy. (default: loadbalance_srcid)'
            - '   - choices: [ loadbalance_ip, loadbalance_srcmac, loadbalance_srcid, loadbalance_loadbased, failover_explicit]'
            - '   - "loadbalance_loadbased" is available from version 2.6 and onwards'
            - '- C(inbound_policy) (bool): Indicate whether or not the teaming policy is applied to inbound frames as well. (default: False)'
            - '- C(notify_switches) (bool): Indicate whether or not to notify the physical switch if a link fails. (default: True)'
            - '- C(rolling_order) (bool): Indicate whether or not to use a rolling policy when restoring links. (default: False)'
        required: False
        version_added: '2.5'
        default: {
            'notify_switches': True,
            'load_balance_policy': 'loadbalance_srcid',
            'inbound_policy': False,
            'rolling_order': False
        }
    port_policy:
        description:
            - Dictionary which configures the advanced policy settings for the portgroup.
            - 'Valid attributes are:'
            - '- C(block_override) (bool): indicates if the block policy can be changed per port. (default: true)'
            - '- C(ipfix_override) (bool): indicates if the ipfix policy can be changed per port. (default: false)'
            - '- C(live_port_move) (bool): indicates if a live port can be moved in or out of the portgroup. (default: false)'
            - '- C(network_rp_override) (bool): indicates if the network resource pool can be changed per port. (default: false)'
            - '- C(port_config_reset_at_disconnect) (bool): indicates if the configuration of a port is reset automatically after disconnect. (default: true)'
            - '- C(security_override) (bool): indicates if the security policy can be changed per port. (default: false)'
            - '- C(shaping_override) (bool): indicates if the shaping policy can be changed per port. (default: false)'
            - '- C(traffic_filter_override) (bool): indicates if the traffic filter can be changed per port. (default: false)'
            - '- C(uplink_teaming_override) (bool): indicates if the uplink teaming policy can be changed per port. (default: false)'
            - '- C(vendor_config_override) (bool): indicates if the vendor config can be changed per port. (default: false)'
            - '- C(vlan_override) (bool): indicates if the vlan can be changed per port. (default: false)'
        required: False
        version_added: '2.5'
        default: {
            'traffic_filter_override': False,
            'network_rp_override': False,
            'live_port_move': False,
            'security_override': False,
            'vendor_config_override': False,
            'port_config_reset_at_disconnect': True,
            'uplink_teaming_override': False,
            'block_override': True,
            'shaping_override': False,
            'vlan_override': False,
            'ipfix_override': False
        }

extends_documentation_fragment: vmware.documentation
'''

EXAMPLES = '''
- name: Create vlan portgroup
  vmware_dvs_portgroup:
    hostname: '{{ vcenter_hostname }}'
    username: '{{ vcenter_username }}'
    password: '{{ vcenter_password }}'
    portgroup_name: vlan-123-portrgoup
    switch_name: dvSwitch
    vlan_id: 123
    num_ports: 120
    portgroup_type: earlyBinding
    state: present
  delegate_to: localhost

- name: Create vlan trunk portgroup
  vmware_dvs_portgroup:
    hostname: '{{ vcenter_hostname }}'
    username: '{{ vcenter_username }}'
    password: '{{ vcenter_password }}'
    portgroup_name: vlan-trunk-portrgoup
    switch_name: dvSwitch
    vlan_id: 1-1000
    vlan_trunk: True
    num_ports: 120
    portgroup_type: earlyBinding
    state: present
  delegate_to: localhost

- name: Create no-vlan portgroup
  vmware_dvs_portgroup:
    hostname: '{{ vcenter_hostname }}'
    username: '{{ vcenter_username }}'
    password: '{{ vcenter_password }}'
    portgroup_name: no-vlan-portrgoup
    switch_name: dvSwitch
    vlan_id: 0
    num_ports: 120
    portgroup_type: earlyBinding
    state: present
  delegate_to: localhost

- name: Create vlan portgroup with all security and port policies
  vmware_dvs_portgroup:
    hostname: '{{ vcenter_hostname }}'
    username: '{{ vcenter_username }}'
    password: '{{ vcenter_password }}'
    portgroup_name: vlan-123-portrgoup
    switch_name: dvSwitch
    vlan_id: 123
    num_ports: 120
    portgroup_type: earlyBinding
    state: present
    network_policy:
      promiscuous: yes
      forged_transmits: yes
      mac_changes: yes
    port_policy:
      block_override: yes
      ipfix_override: yes
      live_port_move: yes
      network_rp_override: yes
      port_config_reset_at_disconnect: yes
      security_override: yes
      shaping_override: yes
      traffic_filter_override: yes
      uplink_teaming_override: yes
      vendor_config_override: yes
      vlan_override: yes
  delegate_to: localhost
'''

try:
    from pyVmomi import vim, vmodl
except ImportError as e:
    pass

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.vmware import (PyVmomi, find_dvs_by_name, find_dvspg_by_name,
                                         vmware_argument_spec, wait_for_task)


class VMwareDvsPortgroup(PyVmomi):
    def __init__(self, module):
        super(VMwareDvsPortgroup, self).__init__(module)
        self.dvs_portgroup = None
        self.switch_name = self.module.params['switch_name']
        self.portgroup_name = self.module.params['portgroup_name']
        self.vlan_id = self.module.params['vlan_id']
        self.num_ports = self.module.params['num_ports']
        self.portgroup_type = self.module.params['portgroup_type']
        self.dv_switch = None
        self.state = self.module.params['state']
        self.vlan_trunk = self.module.params['vlan_trunk']
        self.security_promiscuous = self.module.params['network_policy']['promiscuous']
        self.security_forged_transmits = self.module.params['network_policy']['forged_transmits']
        self.security_mac_changes = self.module.params['network_policy']['mac_changes']
        self.policy_block_override = self.module.params['port_policy']['block_override']
        self.policy_ipfix_override = self.module.params['port_policy']['ipfix_override']
        self.policy_live_port_move = self.module.params['port_policy']['live_port_move']
        self.policy_network_rp_override = self.module.params['port_policy']['network_rp_override']
        self.policy_port_config_reset_at_disconnect = self.module.params['port_policy']['port_config_reset_at_disconnect']
        self.policy_security_override = self.module.params['port_policy']['security_override']
        self.policy_shaping_override = self.module.params['port_policy']['shaping_override']
        self.policy_traffic_filter_override = self.module.params['port_policy']['traffic_filter_override']
        self.policy_uplink_teaming_override = self.module.params['port_policy']['uplink_teaming_override']
        self.policy_vendor_config_override = self.module.params['port_policy']['vendor_config_override']
        self.policy_vlan_override = self.module.params['port_policy']['vlan_override']

    def process_state(self):
        dvspg_states = {
            'absent': {
                'present': self.state_destroy_dvspg,
                'absent': self.state_exit_unchanged,
            },
            'present': {
                'update': self.state_update_dvspg,
                'present': self.state_exit_unchanged,
                'absent': self.state_create_dvspg,
            }
        }
        try:
            dvspg_states[self.state][self.check_dvspg_state()]()
        except vmodl.RuntimeFault as runtime_fault:
            self.module.fail_json(msg=runtime_fault.msg)
        except vmodl.MethodFault as method_fault:
            self.module.fail_json(msg=method_fault.msg)
        except Exception as e:
            self.module.fail_json(msg=str(e))

    def create_port_group(self):
        config = vim.dvs.DistributedVirtualPortgroup.ConfigSpec()

        # Basic config
        config.name = self.portgroup_name
        config.numPorts = self.num_ports

        # Default port config
        config.defaultPortConfig = vim.dvs.VmwareDistributedVirtualSwitch.VmwarePortConfigPolicy()
        if self.vlan_trunk:
            config.defaultPortConfig.vlan = vim.dvs.VmwareDistributedVirtualSwitch.TrunkVlanSpec()
            vlan_id_start, vlan_id_end = self.vlan_id.split('-')
            config.defaultPortConfig.vlan.vlanId = [vim.NumericRange(start=int(vlan_id_start.strip()), end=int(vlan_id_end.strip()))]
        else:
            config.defaultPortConfig.vlan = vim.dvs.VmwareDistributedVirtualSwitch.VlanIdSpec()
            config.defaultPortConfig.vlan.vlanId = int(self.vlan_id)
        config.defaultPortConfig.vlan.inherited = False
        config.defaultPortConfig.securityPolicy = vim.dvs.VmwareDistributedVirtualSwitch.SecurityPolicy()
        config.defaultPortConfig.securityPolicy.allowPromiscuous = vim.BoolPolicy(value=self.security_promiscuous)
        config.defaultPortConfig.securityPolicy.forgedTransmits = vim.BoolPolicy(value=self.security_forged_transmits)
        config.defaultPortConfig.securityPolicy.macChanges = vim.BoolPolicy(value=self.security_mac_changes)

        # Teaming Policy
        teamingPolicy = vim.dvs.VmwareDistributedVirtualSwitch.UplinkPortTeamingPolicy()
        teamingPolicy.policy = vim.StringPolicy(value=self.module.params['teaming_policy']['load_balance_policy'])
        teamingPolicy.reversePolicy = vim.BoolPolicy(value=self.module.params['teaming_policy']['inbound_policy'])
        teamingPolicy.notifySwitches = vim.BoolPolicy(value=self.module.params['teaming_policy']['notify_switches'])
        teamingPolicy.rollingOrder = vim.BoolPolicy(value=self.module.params['teaming_policy']['rolling_order'])
        config.defaultPortConfig.uplinkTeamingPolicy = teamingPolicy

        # PG policy (advanced_policy)
        config.policy = vim.dvs.VmwareDistributedVirtualSwitch.VMwarePortgroupPolicy()
        config.policy.blockOverrideAllowed = self.policy_block_override
        config.policy.ipfixOverrideAllowed = self.policy_ipfix_override
        config.policy.livePortMovingAllowed = self.policy_live_port_move
        config.policy.networkResourcePoolOverrideAllowed = self.policy_network_rp_override
        config.policy.portConfigResetAtDisconnect = self.policy_port_config_reset_at_disconnect
        config.policy.securityPolicyOverrideAllowed = self.policy_security_override
        config.policy.shapingOverrideAllowed = self.policy_shaping_override
        config.policy.trafficFilterOverrideAllowed = self.policy_traffic_filter_override
        config.policy.uplinkTeamingOverrideAllowed = self.policy_uplink_teaming_override
        config.policy.vendorConfigOverrideAllowed = self.policy_vendor_config_override
        config.policy.vlanOverrideAllowed = self.policy_vlan_override

        # PG Type
        config.type = self.portgroup_type

        task = self.dv_switch.AddDVPortgroup_Task([config])
        changed, result = wait_for_task(task)
        return changed, result

    def state_destroy_dvspg(self):
        changed = True
        result = None

        if not self.module.check_mode:
            task = self.dvs_portgroup.Destroy_Task()
            changed, result = wait_for_task(task)
        self.module.exit_json(changed=changed, result=str(result))

    def state_exit_unchanged(self):
        self.module.exit_json(changed=False)

    def state_update_dvspg(self):
        self.module.exit_json(changed=False, msg="Currently not implemented.")

    def state_create_dvspg(self):
        changed = True
        result = None

        if not self.module.check_mode:
            changed, result = self.create_port_group()
        self.module.exit_json(changed=changed, result=str(result))

    def check_dvspg_state(self):
        self.dv_switch = find_dvs_by_name(self.content, self.switch_name)

        if self.dv_switch is None:
            self.module.fail_json(msg="A distributed virtual switch with name %s does not exist" % self.switch_name)
        self.dvs_portgroup = find_dvspg_by_name(self.dv_switch, self.portgroup_name)

        if self.dvs_portgroup is None:
            return 'absent'
        else:
            return 'present'


def main():
    argument_spec = vmware_argument_spec()
    argument_spec.update(
        dict(
            portgroup_name=dict(required=True, type='str'),
            switch_name=dict(required=True, type='str'),
            vlan_id=dict(required=True, type='str'),
            num_ports=dict(required=True, type='int'),
            portgroup_type=dict(required=True, choices=['earlyBinding', 'lateBinding', 'ephemeral'], type='str'),
            state=dict(required=True, choices=['present', 'absent'], type='str'),
            vlan_trunk=dict(type='bool', default=False),
            network_policy=dict(
                type='dict',
                options=dict(
                    promiscuous=dict(type='bool', default=False),
                    forged_transmits=dict(type='bool', default=False),
                    mac_changes=dict(type='bool', default=False)
                ),
                default=dict(
                    promiscuous=False,
                    forged_transmits=False,
                    mac_changes=False
                )
            ),
            teaming_policy=dict(
                type='dict',
                options=dict(
                    inbound_policy=dict(type='bool', default=False),
                    notify_switches=dict(type='bool', default=True),
                    rolling_order=dict(type='bool', default=False),
                    load_balance_policy=dict(type='str',
                                             default='loadbalance_srcid',
                                             choices=[
                                                 'loadbalance_ip',
                                                 'loadbalance_srcmac',
                                                 'loadbalance_srcid',
                                                 'loadbalance_loadbased',
                                                 'failover_explicit',
                                             ],
                                             )
                ),
                default=dict(
                    inbound_policy=False,
                    notify_switches=True,
                    rolling_order=False,
                    load_balance_policy='loadbalance_srcid',
                ),
            ),
            port_policy=dict(
                type='dict',
                options=dict(
                    block_override=dict(type='bool', default=True),
                    ipfix_override=dict(type='bool', default=False),
                    live_port_move=dict(type='bool', default=False),
                    network_rp_override=dict(type='bool', default=False),
                    port_config_reset_at_disconnect=dict(type='bool', default=True),
                    security_override=dict(type='bool', default=False),
                    shaping_override=dict(type='bool', default=False),
                    traffic_filter_override=dict(type='bool', default=False),
                    uplink_teaming_override=dict(type='bool', default=False),
                    vendor_config_override=dict(type='bool', default=False),
                    vlan_override=dict(type='bool', default=False)
                ),
                default=dict(
                    block_override=True,
                    ipfix_override=False,
                    live_port_move=False,
                    network_rp_override=False,
                    port_config_reset_at_disconnect=True,
                    security_override=False,
                    shaping_override=False,
                    traffic_filter_override=False,
                    uplink_teaming_override=False,
                    vendor_config_override=False,
                    vlan_override=False
                )
            )
        )
    )

    module = AnsibleModule(argument_spec=argument_spec,
                           supports_check_mode=True)

    vmware_dvs_portgroup = VMwareDvsPortgroup(module)
    vmware_dvs_portgroup.process_state()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'network'}


DOCUMENTATION = '''
---
module: nxos_udld
extends_documentation_fragment: nxos
version_added: "2.2"
short_description: Manages UDLD global configuration params.
description:
    - Manages UDLD global configuration params.
author:
    - Jason Edelman (@jedelman8)
notes:
    - Tested against NXOSv 7.3.(0)D1(1) on VIRL
    - Module will fail if the udld feature has not been previously enabled.
options:
    aggressive:
        description:
            - Toggles aggressive mode.
        choices: ['enabled','disabled']
    msg_time:
        description:
            - Message time in seconds for UDLD packets or keyword 'default'.
    reset:
        description:
            - Ability to reset all ports shut down by UDLD. 'state' parameter
              cannot be 'absent' when this is present.
        type: bool
        default: 'no'
    state:
        description:
            - Manage the state of the resource. When set to 'absent',
              aggressive and msg_time are set to their default values.
        default: present
        choices: ['present','absent']

'''
EXAMPLES = '''
# ensure udld aggressive mode is globally disabled and se global message interval is 20
- nxos_udld:
    aggressive: disabled
    msg_time: 20
    host: "{{ inventory_hostname }}"
    username: "{{ un }}"
    password: "{{ pwd }}"

# Ensure agg mode is globally enabled and msg time is 15
- nxos_udld:
    aggressive: enabled
    msg_time: 15
    host: "{{ inventory_hostname }}"
    username: "{{ un }}"
    password: "{{ pwd }}"
'''

RETURN = '''
proposed:
    description: k/v pairs of parameters passed into module
    returned: always
    type: dict
    sample: {"aggressive": "enabled", "msg_time": "40"}
existing:
    description:
        - k/v pairs of existing udld configuration
    returned: always
    type: dict
    sample: {"aggressive": "disabled", "msg_time": "15"}
end_state:
    description: k/v pairs of udld configuration after module execution
    returned: always
    type: dict
    sample: {"aggressive": "enabled", "msg_time": "40"}
updates:
    description: command sent to the device
    returned: always
    type: list
    sample: ["udld message-time 40", "udld aggressive"]
changed:
    description: check to see if a change was made on the device
    returned: always
    type: boolean
    sample: true
'''

import re

from ansible.module_utils.network.nxos.nxos import get_config, load_config, run_commands
from ansible.module_utils.network.nxos.nxos import get_capabilities, nxos_argument_spec
from ansible.module_utils.basic import AnsibleModule


PARAM_TO_DEFAULT_KEYMAP = {
    'msg_time': '15',
}


def flatten_list(command_lists):
    flat_command_list = []
    for command in command_lists:
        if isinstance(command, list):
            flat_command_list.extend(command)
        else:
            flat_command_list.append(command)
    return flat_command_list


def apply_key_map(key_map, table):
    new_dict = {}
    for key, value in table.items():
        new_key = key_map.get(key)
        if new_key:
            value = table.get(key)
            if value:
                new_dict[new_key] = str(value)
            else:
                new_dict[new_key] = value
    return new_dict


def get_commands_config_udld_global(delta, reset, existing):
    commands = []
    for param, value in delta.items():
        if param == 'aggressive':
            command = 'udld aggressive' if value == 'enabled' else 'no udld aggressive'
            commands.append(command)
        elif param == 'msg_time':
            if value == 'default':
                if existing.get('msg_time') != PARAM_TO_DEFAULT_KEYMAP.get('msg_time'):
                    commands.append('no udld message-time')
            else:
                commands.append('udld message-time ' + value)
    if reset:
        command = 'udld reset'
        commands.append(command)
    return commands


def get_commands_remove_udld_global(existing):
    commands = []
    if existing.get('aggressive') == 'enabled':
        command = 'no udld aggressive'
        commands.append(command)
    if existing.get('msg_time') != PARAM_TO_DEFAULT_KEYMAP.get('msg_time'):
        command = 'no udld message-time'
        commands.append(command)
    return commands


def get_udld_global(module):
    command = 'show udld global | json'
    udld_table = run_commands(module, [command])[0]

    status = str(udld_table.get('udld-global-mode', None))
    if status == 'enabled-aggressive':
        aggressive = 'enabled'
    else:
        aggressive = 'disabled'

    interval = str(udld_table.get('message-interval', None))
    udld = dict(msg_time=interval, aggressive=aggressive)

    return udld


def main():
    argument_spec = dict(
        aggressive=dict(required=False, choices=['enabled', 'disabled']),
        msg_time=dict(required=False, type='str'),
        reset=dict(required=False, type='bool'),
        state=dict(choices=['absent', 'present'], default='present'),
    )

    argument_spec.update(nxos_argument_spec)

    module = AnsibleModule(argument_spec=argument_spec,
                           supports_check_mode=True)

    warnings = list()

    aggressive = module.params['aggressive']
    msg_time = module.params['msg_time']
    reset = module.params['reset']
    state = module.params['state']

    if reset and state == 'absent':
        module.fail_json(msg="state must be present when using reset flag.")

    args = dict(aggressive=aggressive, msg_time=msg_time, reset=reset)
    proposed = dict((k, v) for k, v in args.items() if v is not None)

    existing = get_udld_global(module)
    end_state = existing

    delta = set(proposed.items()).difference(existing.items())
    changed = False

    commands = []
    if state == 'present':
        if delta:
            command = get_commands_config_udld_global(dict(delta), reset, existing)
            commands.append(command)

    elif state == 'absent':
        command = get_commands_remove_udld_global(existing)
        if command:
            commands.append(command)

    cmds = flatten_list(commands)
    if cmds:
        if module.check_mode:
            module.exit_json(changed=True, commands=cmds)
        else:
            changed = True
            load_config(module, cmds)
            end_state = get_udld_global(module)
            if 'configure' in cmds:
                cmds.pop(0)

    results = {}
    results['proposed'] = proposed
    results['existing'] = existing
    results['end_state'] = end_state
    results['updates'] = cmds
    results['changed'] = changed
    results['warnings'] = warnings

    module.exit_json(**results)


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'network'}


DOCUMENTATION = """
---
module: eos_user
version_added: "2.3"
author: "Peter Sprygada (@privateip)"
short_description: Manage the collection of local users on EOS devices
description:
  - This module provides declarative management of the local usernames
    configured on Arista EOS devices.  It allows playbooks to manage
    either individual usernames or the collection of usernames in the
    current running config.  It also supports purging usernames from the
    configuration that are not explicitly defined.
extends_documentation_fragment: eos
notes:
  - Tested against EOS 4.15
options:
  aggregate:
    description:
      - The set of username objects to be configured on the remote
        Arista EOS device.  The list entries can either be the username
        or a hash of username and properties.  This argument is mutually
        exclusive with the C(username) argument.
    aliases: ['users', 'collection']
    version_added: "2.4"
  name:
    description:
      - The username to be configured on the remote Arista EOS
        device.  This argument accepts a stringv value and is mutually
        exclusive with the C(aggregate) argument.
        Please note that this option is not same as C(provider username).
    version_added: "2.4"
  configured_password:
    description:
      - The password to be configured on the remote Arista EOS device. The
        password needs to be provided in clear and it will be encrypted
        on the device.
        Please note that this option is not same as C(provider password).
    version_added: "2.4"
  update_password:
    description:
      - Since passwords are encrypted in the device running config, this
        argument will instruct the module when to change the password.  When
        set to C(always), the password will always be updated in the device
        and when set to C(on_create) the password will be updated only if
        the username is created.
    default: always
    choices: ['on_create', 'always']
  privilege:
    description:
      - The C(privilege) argument configures the privilege level of the
        user when logged into the system.  This argument accepts integer
        values in the range of 1 to 15.
  role:
    description:
      - Configures the role for the username in the
        device running configuration.  The argument accepts a string value
        defining the role name.  This argument does not check if the role
        has been configured on the device.
  sshkey:
    description:
      - Specifies the SSH public key to configure
        for the given username.  This argument accepts a valid SSH key value.
  nopassword:
    description:
      - Defines the username without assigning
        a password.  This will allow the user to login to the system
        without being authenticated by a password.
    type: bool
  purge:
    description:
      - Instructs the module to consider the
        resource definition absolute.  It will remove any previously
        configured usernames on the device with the exception of the
        `admin` user which cannot be deleted per EOS constraints.
    type: bool
    default: false
  state:
    description:
      - Configures the state of the username definition
        as it relates to the device operational configuration.  When set
        to I(present), the username(s) should be configured in the device active
        configuration and when set to I(absent) the username(s) should not be
        in the device active configuration
    default: present
    choices: ['present', 'absent']
"""

EXAMPLES = """
- name: create a new user
  eos_user:
    name: ansible
    sshkey: "{{ lookup('file', '~/.ssh/id_rsa.pub') }}"
    state: present

- name: remove all users except admin
  eos_user:
    purge: yes

- name: set multiple users to privilege level 15
  eos_user:
    aggregate:
      - name: netop
      - name: netend
    privilege: 15
    state: present

- name: Change Password for User netop
  eos_user:
    username: netop
    configured_password: "{{ new_password }}"
    update_password: always
    state: present
"""

RETURN = """
commands:
  description: The list of configuration mode commands to send to the device
  returned: always
  type: list
  sample:
    - name ansible secret password
    - name admin secret admin
session_name:
  description: The EOS config session name used to load the configuration
  returned: when changed is True
  type: str
  sample: ansible_1479315771
"""

import re

from copy import deepcopy
from functools import partial

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.network.common.utils import remove_default_spec
from ansible.module_utils.network.eos.eos import get_config, load_config
from ansible.module_utils.network.eos.eos import eos_argument_spec, check_args
from ansible.module_utils.six import iteritems


def validate_privilege(value, module):
    if not 1 <= value <= 15:
        module.fail_json(msg='privilege must be between 1 and 15, got %s' % value)


def map_obj_to_commands(updates, module):
    commands = list()
    state = module.params['state']
    update_password = module.params['update_password']

    for update in updates:
        want, have = update

        def needs_update(x):
            return want.get(x) and (want.get(x) != have.get(x))

        def add(x):
            return commands.append('username %s %s' % (want['name'], x))

        if want['state'] == 'absent':
            commands.append('no username %s' % want['name'])
            continue

        if needs_update('configured_password'):
            if update_password == 'always' or not have:
                add('secret %s' % want['configured_password'])

        if needs_update('role'):
            add('role %s' % want['role'])

        if needs_update('privilege'):
            add('privilege %s' % want['privilege'])

        if needs_update('sshkey'):
            add('sshkey %s' % want['sshkey'])

        if needs_update('nopassword'):
            if want['nopassword']:
                add('nopassword')
            else:
                add('no username %s nopassword' % want['name'])

        if want.get('state') == 'present' and want.get('name'):
            value = [want.get('configured_password'), want.get('nopassword'), want.get('sshkey')]
            if all(v is None for v in value) is True:
                module.fail_json(msg='configured_password, sshkey or nopassword should be provided')

    return commands


def parse_role(data):
    match = re.search(r'role (\S+)', data, re.M)
    if match:
        return match.group(1)


def parse_sshkey(data):
    match = re.search(r'sshkey (.+)$', data, re.M)
    if match:
        return match.group(1)


def parse_privilege(data):
    match = re.search(r'privilege (\S+)', data, re.M)
    if match:
        return int(match.group(1))


def map_config_to_obj(module):
    data = get_config(module, flags=['section username'])

    match = re.findall(r'^username (\S+)', data, re.M)
    if not match:
        return list()

    instances = list()

    for user in set(match):
        regex = r'username %s .+$' % user
        cfg = re.findall(regex, data, re.M)
        cfg = '\n'.join(cfg)
        obj = {
            'name': user,
            'state': 'present',
            'nopassword': 'nopassword' in cfg,
            'configured_password': None,
            'sshkey': parse_sshkey(cfg),
            'privilege': parse_privilege(cfg),
            'role': parse_role(cfg)
        }
        instances.append(obj)

    return instances


def get_param_value(key, item, module):
    # if key doesn't exist in the item, get it from module.params
    if not item.get(key):
        value = module.params[key]

    # if key does exist, do a type check on it to validate it
    else:
        value_type = module.argument_spec[key].get('type', 'str')
        type_checker = module._CHECK_ARGUMENT_TYPES_DISPATCHER[value_type]
        type_checker(item[key])
        value = item[key]

    # validate the param value (if validator func exists)
    validator = globals().get('validate_%s' % key)
    if all((value, validator)):
        validator(value, module)

    return value


def map_params_to_obj(module):
    aggregate = module.params['aggregate']
    if not aggregate:
        if not module.params['name'] and module.params['purge']:
            return list()
        elif not module.params['name']:
            module.fail_json(msg='name is required')
        else:
            collection = [{'name': module.params['name']}]
    else:
        collection = list()
        for item in aggregate:
            if not isinstance(item, dict):
                collection.append({'name': item})
            elif 'name' not in item:
                module.fail_json(msg='name is required')
            else:
                collection.append(item)

    objects = list()

    for item in collection:
        get_value = partial(get_param_value, item=item, module=module)
        item['configured_password'] = get_value('configured_password')
        item['nopassword'] = get_value('nopassword')
        item['privilege'] = get_value('privilege')
        item['role'] = get_value('role')
        item['sshkey'] = get_value('sshkey')
        item['state'] = get_value('state')
        objects.append(item)

    return objects


def update_objects(want, have):
    updates = list()
    for entry in want:
        if 'name' in entry:
            item = next((i for i in have if i['name'] == entry['name']), None)
        if all((item is None, entry['state'] == 'present')):
            updates.append((entry, {}))
        elif item:
            for key, value in iteritems(entry):
                if value and value != item[key]:
                    updates.append((entry, item))
    return updates


def main():
    """ main entry point for module execution
    """
    element_spec = dict(
        name=dict(),

        configured_password=dict(no_log=True),
        nopassword=dict(type='bool'),
        update_password=dict(default='always', choices=['on_create', 'always']),

        privilege=dict(type='int'),
        role=dict(),

        sshkey=dict(),

        state=dict(default='present', choices=['present', 'absent'])
    )

    aggregate_spec = deepcopy(element_spec)

    # remove default in aggregate spec, to handle common arguments
    remove_default_spec(aggregate_spec)

    argument_spec = dict(
        aggregate=dict(type='list', elements='dict', options=aggregate_spec, aliases=['collection', 'users']),
        purge=dict(type='bool', default=False)
    )

    argument_spec.update(element_spec)
    argument_spec.update(eos_argument_spec)
    mutually_exclusive = [('name', 'aggregate')]

    module = AnsibleModule(argument_spec=argument_spec,
                           mutually_exclusive=mutually_exclusive,
                           supports_check_mode=True)

    warnings = list()
    if module.params['password'] and not module.params['configured_password']:
        warnings.append(
            'The "password" argument is used to authenticate the current connection. ' +
            'To set a user password use "configured_password" instead.'
        )

    check_args(module, warnings)

    result = {'changed': False}
    if warnings:
        result['warnings'] = warnings

    want = map_params_to_obj(module)
    have = map_config_to_obj(module)

    commands = map_obj_to_commands(update_objects(want, have), module)

    if module.params['purge']:
        want_users = [x['name'] for x in want]
        have_users = [x['name'] for x in have]
        for item in set(have_users).difference(want_users):
            if item != 'admin':
                commands.append('no username %s' % item)

    result['commands'] = commands

    # the eos cli prevents this by rule so capture it and display
    # a nice failure message
    if 'no username admin' in commands:
        module.fail_json(msg='cannot delete the `admin` account')

    if commands:
        commit = not module.check_mode
        response = load_config(module, commands, commit=commit)
        if response.get('diff') and module._diff:
            result['diff'] = {'prepared': response.get('diff')}
        result['session_name'] = response.get('session')
        result['changed'] = True

    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2014, Nate Coraor <nate@bx.psu.edu>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = '''
---
module: capabilities
short_description: Manage Linux capabilities
description:
    - This module manipulates files privileges using the Linux capabilities(7) system.
version_added: "1.6"
options:
    path:
        description:
            - Specifies the path to the file to be managed.
        required: yes
    capability:
        description:
            - Desired capability to set (with operator and flags, if state is C(present)) or remove (if state is C(absent))
        required: yes
        aliases: [ cap ]
    state:
        description:
            - Whether the entry should be present or absent in the file's capabilities.
        choices: [ absent, present ]
        default: present
notes:
    - The capabilities system will automatically transform operators and flags
      into the effective set, so (for example, cap_foo=ep will probably become
      cap_foo+ep). This module does not attempt to determine the final operator
      and flags to compare, so you will want to ensure that your capabilities
      argument matches the final capabilities.
author:
- Nate Coraor (@natefoo)
'''

EXAMPLES = '''
- name: Set cap_sys_chroot+ep on /foo
  capabilities:
    path: /foo
    capability: cap_sys_chroot+ep
    state: present

- name: Remove cap_net_bind_service from /bar
  capabilities:
    path: /bar
    capability: cap_net_bind_service
    state: absent
'''

from ansible.module_utils.basic import AnsibleModule

OPS = ('=', '-', '+')


class CapabilitiesModule(object):
    platform = 'Linux'
    distribution = None

    def __init__(self, module):
        self.module = module
        self.path = module.params['path'].strip()
        self.capability = module.params['capability'].strip().lower()
        self.state = module.params['state']
        self.getcap_cmd = module.get_bin_path('getcap', required=True)
        self.setcap_cmd = module.get_bin_path('setcap', required=True)
        self.capability_tup = self._parse_cap(self.capability, op_required=self.state == 'present')

        self.run()

    def run(self):

        current = self.getcap(self.path)
        caps = [cap[0] for cap in current]

        if self.state == 'present' and self.capability_tup not in current:
            # need to add capability
            if self.module.check_mode:
                self.module.exit_json(changed=True, msg='capabilities changed')
            else:
                # remove from current cap list if it's already set (but op/flags differ)
                current = list(filter(lambda x: x[0] != self.capability_tup[0], current))
                # add new cap with correct op/flags
                current.append(self.capability_tup)
                self.module.exit_json(changed=True, state=self.state, msg='capabilities changed', stdout=self.setcap(self.path, current))
        elif self.state == 'absent' and self.capability_tup[0] in caps:
            # need to remove capability
            if self.module.check_mode:
                self.module.exit_json(changed=True, msg='capabilities changed')
            else:
                # remove from current cap list and then set current list
                current = filter(lambda x: x[0] != self.capability_tup[0], current)
                self.module.exit_json(changed=True, state=self.state, msg='capabilities changed', stdout=self.setcap(self.path, current))
        self.module.exit_json(changed=False, state=self.state)

    def getcap(self, path):
        rval = []
        cmd = "%s -v %s" % (self.getcap_cmd, path)
        rc, stdout, stderr = self.module.run_command(cmd)
        # If file xattrs are set but no caps are set the output will be:
        #   '/foo ='
        # If file xattrs are unset the output will be:
        #   '/foo'
        # If the file does not eixst the output will be (with rc == 0...):
        #   '/foo (No such file or directory)'
        if rc != 0 or (stdout.strip() != path and stdout.count(' =') != 1):
            self.module.fail_json(msg="Unable to get capabilities of %s" % path, stdout=stdout.strip(), stderr=stderr)
        if stdout.strip() != path:
            caps = stdout.split(' =')[1].strip().split()
            for cap in caps:
                cap = cap.lower()
                # getcap condenses capabilities with the same op/flags into a
                # comma-separated list, so we have to parse that
                if ',' in cap:
                    cap_group = cap.split(',')
                    cap_group[-1], op, flags = self._parse_cap(cap_group[-1])
                    for subcap in cap_group:
                        rval.append((subcap, op, flags))
                else:
                    rval.append(self._parse_cap(cap))
        return rval

    def setcap(self, path, caps):
        caps = ' '.join([''.join(cap) for cap in caps])
        cmd = "%s '%s' %s" % (self.setcap_cmd, caps, path)
        rc, stdout, stderr = self.module.run_command(cmd)
        if rc != 0:
            self.module.fail_json(msg="Unable to set capabilities of %s" % path, stdout=stdout, stderr=stderr)
        else:
            return stdout

    def _parse_cap(self, cap, op_required=True):
        opind = -1
        try:
            i = 0
            while opind == -1:
                opind = cap.find(OPS[i])
                i += 1
        except:
            if op_required:
                self.module.fail_json(msg="Couldn't find operator (one of: %s)" % str(OPS))
            else:
                return (cap, None, None)
        op = cap[opind]
        cap, flags = cap.split(op)
        return (cap, op, flags)


# ==============================================================
# main

def main():
    # defining module
    module = AnsibleModule(
        argument_spec=dict(
            path=dict(type='str', required=True, aliases=['key']),
            capability=dict(type='str', required=True, aliases=['cap']),
            state=dict(type='str', default='present', choices=['absent', 'present']),
        ),
        supports_check_mode=True,
    )

    CapabilitiesModule(module)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017 Google
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
# ----------------------------------------------------------------------------
#
#     ***     AUTO GENERATED CODE    ***    AUTO GENERATED CODE     ***
#
# ----------------------------------------------------------------------------
#
#     This file is automatically generated by Magic Modules and manual
#     changes will be clobbered when the file is regenerated.
#
#     Please read more about how to change this file at
#     https://www.github.com/GoogleCloudPlatform/magic-modules
#
# ----------------------------------------------------------------------------

from __future__ import absolute_import, division, print_function
__metaclass__ = type

################################################################################
# Documentation
################################################################################

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ["preview"],
                    'supported_by': 'community'}

DOCUMENTATION = '''
---
module: gcp_compute_ssl_policy
description:
    - Represents a SSL policy. SSL policies give you the ability to control the features
      of SSL that your SSL proxy or HTTPS load balancer negotiates.
short_description: Creates a GCP SslPolicy
version_added: 2.7
author: Google Inc. (@googlecloudplatform)
requirements:
    - python >= 2.6
    - requests >= 2.18.4
    - google-auth >= 1.3.0
options:
    state:
        description:
            - Whether the given object should exist in GCP
        choices: ['present', 'absent']
        default: 'present'
    description:
        description:
            - An optional description of this resource.
        required: false
    name:
        description:
            - Name of the resource. Provided by the client when the resource is created. The name
              must be 1-63 characters long, and comply with RFC1035. Specifically, the name must
              be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?`
              which means the first character must be a lowercase letter, and all following characters
              must be a dash, lowercase letter, or digit, except the last character, which cannot
              be a dash.
        required: true
    profile:
        description:
            - Profile specifies the set of SSL features that can be used by the load balancer
              when negotiating SSL with clients. This can be one of `COMPATIBLE`, `MODERN`, `RESTRICTED`,
              or `CUSTOM`. If using `CUSTOM`, the set of SSL features to enable must be specified
              in the `customFeatures` field.
        required: false
        choices: ['COMPATIBLE', 'MODERN', 'RESTRICTED', 'CUSTOM']
    min_tls_version:
        description:
            - The minimum version of SSL protocol that can be used by the clients to establish
              a connection with the load balancer. This can be one of `TLS_1_0`, `TLS_1_1`, `TLS_1_2`.
        required: false
        choices: ['TLS_1_0', 'TLS_1_1', 'TLS_1_2']
    custom_features:
        description:
            - A list of features enabled when the selected profile is CUSTOM. The method returns
              the set of features that can be specified in this list. This field must be empty
              if the profile is not CUSTOM.
        required: false
extends_documentation_fragment: gcp
notes:
    - "API Reference: U(https://cloud.google.com/compute/docs/reference/rest/v1/sslPolicies)"
    - "Using SSL Policies: U(https://cloud.google.com/compute/docs/load-balancing/ssl-policies)"
'''

EXAMPLES = '''
- name: create a ssl policy
  gcp_compute_ssl_policy:
      name: "test_object"
      profile: CUSTOM
      min_tls_version: TLS_1_2
      custom_features:
      - TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384
      - TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384
      project: "test_project"
      auth_kind: "service_account"
      service_account_file: "/tmp/auth.pem"
      state: present
'''

RETURN = '''
    creation_timestamp:
        description:
            - Creation timestamp in RFC3339 text format.
        returned: success
        type: str
    description:
        description:
            - An optional description of this resource.
        returned: success
        type: str
    id:
        description:
            - The unique identifier for the resource.
        returned: success
        type: int
    name:
        description:
            - Name of the resource. Provided by the client when the resource is created. The name
              must be 1-63 characters long, and comply with RFC1035. Specifically, the name must
              be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?`
              which means the first character must be a lowercase letter, and all following characters
              must be a dash, lowercase letter, or digit, except the last character, which cannot
              be a dash.
        returned: success
        type: str
    profile:
        description:
            - Profile specifies the set of SSL features that can be used by the load balancer
              when negotiating SSL with clients. This can be one of `COMPATIBLE`, `MODERN`, `RESTRICTED`,
              or `CUSTOM`. If using `CUSTOM`, the set of SSL features to enable must be specified
              in the `customFeatures` field.
        returned: success
        type: str
    min_tls_version:
        description:
            - The minimum version of SSL protocol that can be used by the clients to establish
              a connection with the load balancer. This can be one of `TLS_1_0`, `TLS_1_1`, `TLS_1_2`.
        returned: success
        type: str
    enabled_features:
        description:
            - The list of features enabled in the SSL policy.
        returned: success
        type: list
    custom_features:
        description:
            - A list of features enabled when the selected profile is CUSTOM. The method returns
              the set of features that can be specified in this list. This field must be empty
              if the profile is not CUSTOM.
        returned: success
        type: list
    fingerprint:
        description:
            - Fingerprint of this resource. A hash of the contents stored in this object. This
              field is used in optimistic locking.
        returned: success
        type: str
    warnings:
        description:
            - If potential misconfigurations are detected for this SSL policy, this field will
              be populated with warning messages.
        returned: success
        type: complex
        contains:
            code:
                description:
                    - A warning code, if applicable.
                returned: success
                type: str
            message:
                description:
                    - A human-readable description of the warning code.
                returned: success
                type: str
'''

################################################################################
# Imports
################################################################################

from ansible.module_utils.gcp_utils import navigate_hash, GcpSession, GcpModule, GcpRequest, remove_nones_from_dict, replace_resource_dict
import json
import time

################################################################################
# Main
################################################################################


def main():
    """Main function"""

    module = GcpModule(
        argument_spec=dict(
            state=dict(default='present', choices=['present', 'absent'], type='str'),
            description=dict(type='str'),
            name=dict(required=True, type='str'),
            profile=dict(type='str', choices=['COMPATIBLE', 'MODERN', 'RESTRICTED', 'CUSTOM']),
            min_tls_version=dict(type='str', choices=['TLS_1_0', 'TLS_1_1', 'TLS_1_2']),
            custom_features=dict(type='list', elements='str')
        )
    )

    if not module.params['scopes']:
        module.params['scopes'] = ['https://www.googleapis.com/auth/compute']

    state = module.params['state']
    kind = 'compute#sslPolicy'

    fetch = fetch_resource(module, self_link(module), kind)
    changed = False

    if fetch:
        if state == 'present':
            if is_different(module, fetch):
                fetch = update(module, self_link(module), kind)
                changed = True
        else:
            delete(module, self_link(module), kind)
            fetch = {}
            changed = True
    else:
        if state == 'present':
            fetch = create(module, collection(module), kind)
            changed = True
        else:
            fetch = {}

    fetch.update({'changed': changed})

    module.exit_json(**fetch)


def create(module, link, kind):
    auth = GcpSession(module, 'compute')
    return wait_for_operation(module, auth.post(link, resource_to_request(module)))


def update(module, link, kind):
    auth = GcpSession(module, 'compute')
    return wait_for_operation(module, auth.patch(link, resource_to_request(module)))


def delete(module, link, kind):
    auth = GcpSession(module, 'compute')
    return wait_for_operation(module, auth.delete(link))


def resource_to_request(module):
    request = {
        u'kind': 'compute#sslPolicy',
        u'description': module.params.get('description'),
        u'name': module.params.get('name'),
        u'profile': module.params.get('profile'),
        u'minTlsVersion': module.params.get('min_tls_version'),
        u'customFeatures': module.params.get('custom_features')
    }
    return_vals = {}
    for k, v in request.items():
        if v:
            return_vals[k] = v

    return return_vals


def fetch_resource(module, link, kind):
    auth = GcpSession(module, 'compute')
    return return_if_object(module, auth.get(link), kind)


def self_link(module):
    return "https://www.googleapis.com/compute/v1/projects/{project}/global/sslPolicies/{name}".format(**module.params)


def collection(module):
    return "https://www.googleapis.com/compute/v1/projects/{project}/global/sslPolicies".format(**module.params)


def return_if_object(module, response, kind):
    # If not found, return nothing.
    if response.status_code == 404:
        return None

    # If no content, return nothing.
    if response.status_code == 204:
        return None

    try:
        module.raise_for_status(response)
        result = response.json()
    except getattr(json.decoder, 'JSONDecodeError', ValueError) as inst:
        module.fail_json(msg="Invalid JSON response with error: %s" % inst)

    if navigate_hash(result, ['error', 'errors']):
        module.fail_json(msg=navigate_hash(result, ['error', 'errors']))
    if result['kind'] != kind:
        module.fail_json(msg="Incorrect result: {kind}".format(**result))

    return result


def is_different(module, response):
    request = resource_to_request(module)
    response = response_to_hash(module, response)

    # Remove all output-only from response.
    response_vals = {}
    for k, v in response.items():
        if k in request:
            response_vals[k] = v

    request_vals = {}
    for k, v in request.items():
        if k in response:
            request_vals[k] = v

    return GcpRequest(request_vals) != GcpRequest(response_vals)


# Remove unnecessary properties from the response.
# This is for doing comparisons with Ansible's current parameters.
def response_to_hash(module, response):
    return {
        u'creationTimestamp': response.get(u'creationTimestamp'),
        u'description': module.params.get('description'),
        u'id': response.get(u'id'),
        u'name': module.params.get('name'),
        u'profile': response.get(u'profile'),
        u'minTlsVersion': response.get(u'minTlsVersion'),
        u'enabledFeatures': response.get(u'enabledFeatures'),
        u'customFeatures': response.get(u'customFeatures'),
        u'fingerprint': response.get(u'fingerprint'),
        u'warnings': SslPolicyWarningsArray(response.get(u'warnings', []), module).from_response()
    }


def async_op_url(module, extra_data=None):
    if extra_data is None:
        extra_data = {}
    url = "https://www.googleapis.com/compute/v1/projects/{project}/global/operations/{op_id}"
    combined = extra_data.copy()
    combined.update(module.params)
    return url.format(**combined)


def wait_for_operation(module, response):
    op_result = return_if_object(module, response, 'compute#operation')
    if op_result is None:
        return {}
    status = navigate_hash(op_result, ['status'])
    wait_done = wait_for_completion(status, op_result, module)
    return fetch_resource(module, navigate_hash(wait_done, ['targetLink']), 'compute#sslPolicy')


def wait_for_completion(status, op_result, module):
    op_id = navigate_hash(op_result, ['name'])
    op_uri = async_op_url(module, {'op_id': op_id})
    while status != 'DONE':
        raise_if_errors(op_result, ['error', 'errors'], 'message')
        time.sleep(1.0)
        if status not in ['PENDING', 'RUNNING', 'DONE']:
            module.fail_json(msg="Invalid result %s" % status)
        op_result = fetch_resource(module, op_uri, 'compute#operation')
        status = navigate_hash(op_result, ['status'])
    return op_result


def raise_if_errors(response, err_path, module):
    errors = navigate_hash(response, err_path)
    if errors is not None:
        module.fail_json(msg=errors)


class SslPolicyWarningsArray(object):
    def __init__(self, request, module):
        self.module = module
        if request:
            self.request = request
        else:
            self.request = []

    def to_request(self):
        items = []
        for item in self.request:
            items.append(self._request_for_item(item))
        return items

    def from_response(self):
        items = []
        for item in self.request:
            items.append(self._response_from_item(item))
        return items

    def _request_for_item(self, item):
        return remove_nones_from_dict({
            u'code': item.get('code'),
            u'message': item.get('message')
        })

    def _response_from_item(self, item):
        return remove_nones_from_dict({
            u'code': item.get(u'code'),
            u'message': item.get(u'message')
        })


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'network'}


DOCUMENTATION = '''
---
module: nxos_snmp_host
extends_documentation_fragment: nxos
version_added: "2.2"
short_description: Manages SNMP host configuration.
description:
    - Manages SNMP host configuration parameters.
author:
    - Jason Edelman (@jedelman8)
    - Gabriele Gerbino (@GGabriele)
notes:
    - Tested against NXOSv 7.3.(0)D1(1) on VIRL
    - C(state=absent) removes the host configuration if it is configured.
options:
    snmp_host:
        description:
            - IP address of hostname of target host.
        required: true
    version:
        description:
            - SNMP version. If this is not specified, v1 is used.
        choices: ['v1', 'v2c', 'v3']
    v3:
        description:
            - Use this when verion is v3. SNMPv3 Security level.
        choices: ['noauth', 'auth', 'priv']
    community:
        description:
            - Community string or v3 username.
    udp:
        description:
            - UDP port number (0-65535).
        default: 162
    snmp_type:
        description:
            - type of message to send to host. If this is not
              specified, trap type is used.
        choices: ['trap', 'inform']
    vrf:
        description:
            - VRF to use to source traffic to source.
              If state = absent, the vrf is removed.
    vrf_filter:
        description:
            - Name of VRF to filter.
              If state = absent, the vrf is removed from the filter.
    src_intf:
        description:
            - Source interface. Must be fully qualified interface name.
              If state = absent, the interface is removed.
    state:
        description:
            - Manage the state of the resource. If state = present, the
              host is added to the configuration. If only vrf and/or
              vrf_filter and/or src_intf are given, they will be added to
              the existing host configuration. If state = absent, the
              host is removed if community parameter is given. It is possible
              to remove only vrf and/or src_int and/or vrf_filter
              by providing only those parameters and no community parameter.
        default: present
        choices: ['present','absent']
'''

EXAMPLES = '''
# ensure snmp host is configured
- nxos_snmp_host:
    snmp_host: 192.0.2.3
    community: TESTING
    state: present
'''

RETURN = '''
commands:
    description: commands sent to the device
    returned: always
    type: list
    sample: ["snmp-server host 192.0.2.3 filter-vrf another_test_vrf"]
'''


import re
from ansible.module_utils.network.nxos.nxos import load_config, run_commands
from ansible.module_utils.network.nxos.nxos import nxos_argument_spec, check_args
from ansible.module_utils.basic import AnsibleModule


def execute_show_command(command, module):
    command = {
        'command': command,
        'output': 'json',
    }

    return run_commands(module, command)


def apply_key_map(key_map, table):
    new_dict = {}
    for key, value in table.items():
        new_key = key_map.get(key)
        if new_key:
            value = table.get(key)
            if value:
                new_dict[new_key] = str(value)
            else:
                new_dict[new_key] = value
    return new_dict


def flatten_list(command_lists):
    flat_command_list = []
    for command in command_lists:
        if isinstance(command, list):
            flat_command_list.extend(command)
        else:
            flat_command_list.append(command)
    return flat_command_list


def get_snmp_host(host, udp, module):
    body = execute_show_command('show snmp host', module)

    host_map = {
        'port': 'udp',
        'version': 'version',
        'level': 'v3',
        'type': 'snmp_type',
        'secname': 'community'
    }

    host_map_5k = {
        'port': 'udp',
        'version': 'version',
        'sec_level': 'v3',
        'notif_type': 'snmp_type',
        'commun_or_user': 'community'
    }

    resource = {}

    if body:
        try:
            resource_table = body[0]['TABLE_host']['ROW_host']

            if isinstance(resource_table, dict):
                resource_table = [resource_table]

            for each in resource_table:
                key = str(each['host']) + '_' + str(each['port']).strip()
                src = each.get('src_intf')
                host_resource = apply_key_map(host_map, each)

                if src:
                    host_resource['src_intf'] = src
                    if re.search(r'interface:', src):
                        host_resource['src_intf'] = src.split(':')[1].strip()

                vrf_filt = each.get('TABLE_vrf_filters')
                if vrf_filt:
                    vrf_filter = vrf_filt['ROW_vrf_filters']['vrf_filter'].split(':')[1].split(',')
                    filters = [vrf.strip() for vrf in vrf_filter]
                    host_resource['vrf_filter'] = filters

                vrf = each.get('vrf')
                if vrf:
                    host_resource['vrf'] = vrf.split(':')[1].strip()
                resource[key] = host_resource
        except KeyError:
            # Handle the 5K case
            try:
                resource_table = body[0]['TABLE_hosts']['ROW_hosts']

                if isinstance(resource_table, dict):
                    resource_table = [resource_table]

                for each in resource_table:
                    key = str(each['address']) + '_' + str(each['port']).strip()
                    src = each.get('src_intf')
                    host_resource = apply_key_map(host_map_5k, each)

                    if src:
                        host_resource['src_intf'] = src
                        if re.search(r'interface:', src):
                            host_resource['src_intf'] = src.split(':')[1].strip()

                    vrf = each.get('use_vrf_name')
                    if vrf:
                        host_resource['vrf'] = vrf.strip()

                    vrf_filt = each.get('TABLE_filter_vrf')
                    if vrf_filt:
                        vrf_filter = vrf_filt['ROW_filter_vrf']['filter_vrf_name'].split(',')
                        filters = [vrf.strip() for vrf in vrf_filter]
                        host_resource['vrf_filter'] = filters

                    resource[key] = host_resource
            except (KeyError, AttributeError, TypeError):
                return resource
        except (AttributeError, TypeError):
            return resource

        find = resource.get(host + '_' + udp)

        if find:
            fix_find = {}
            for (key, value) in find.items():
                if isinstance(value, str):
                    fix_find[key] = value.strip()
                else:
                    fix_find[key] = value
            return fix_find

    return {}


def remove_snmp_host(host, udp, existing):
    commands = []
    if existing['version'] == 'v3':
        existing['version'] = '3'
        command = 'no snmp-server host {0} {snmp_type} version \
                    {version} {v3} {community} udp-port {1}'.format(host, udp, **existing)

    elif existing['version'] == 'v2c':
        existing['version'] = '2c'
        command = 'no snmp-server host {0} {snmp_type} version \
                    {version} {community} udp-port {1}'.format(host, udp, **existing)

    elif existing['version'] == 'v1':
        existing['version'] = '1'
        command = 'no snmp-server host {0} {snmp_type} version \
                    {version} {community} udp-port {1}'.format(host, udp, **existing)

    if command:
        commands.append(command)
    return commands


def remove_vrf(host, udp, proposed, existing):
    commands = []
    if existing.get('vrf'):
        commands.append('no snmp-server host {0} use-vrf \
                    {1} udp-port {2}'.format(host, proposed.get('vrf'), udp))
    return commands


def remove_filter(host, udp, proposed, existing):
    commands = []
    if existing.get('vrf_filter'):
        if proposed.get('vrf_filter') in existing.get('vrf_filter'):
            commands.append('no snmp-server host {0} filter-vrf \
                    {1} udp-port {2}'.format(host, proposed.get('vrf_filter'), udp))
    return commands


def remove_src(host, udp, proposed, existing):
    commands = []
    if existing.get('src_intf'):
        commands.append('no snmp-server host {0} source-interface \
                    {1} udp-port {2}'.format(host, proposed.get('src_intf'), udp))
    return commands


def config_snmp_host(delta, udp, proposed, existing, module):
    commands = []
    command_builder = []
    host = proposed['snmp_host']
    cmd = 'snmp-server host {0}'.format(proposed['snmp_host'])

    snmp_type = delta.get('snmp_type')
    version = delta.get('version')
    ver = delta.get('v3')
    community = delta.get('community')

    command_builder.append(cmd)
    if any([snmp_type, version, ver, community]):
        type_string = snmp_type or existing.get('type')
        if type_string:
            command_builder.append(type_string)

        version = version or existing.get('version')
        if version:
            if version == 'v1':
                vn = '1'
            elif version == 'v2c':
                vn = '2c'
            elif version == 'v3':
                vn = '3'

            version_string = 'version {0}'.format(vn)
            command_builder.append(version_string)

        if ver:
            ver_string = ver or existing.get('v3')
            command_builder.append(ver_string)

        if community:
            community_string = community or existing.get('community')
            command_builder.append(community_string)

        udp_string = ' udp-port {0}'.format(udp)
        command_builder.append(udp_string)

        cmd = ' '.join(command_builder)

        commands.append(cmd)

    CMDS = {
        'vrf_filter': 'snmp-server host {0} filter-vrf {vrf_filter} udp-port {1}',
        'vrf': 'snmp-server host {0} use-vrf {vrf} udp-port {1}',
        'src_intf': 'snmp-server host {0} source-interface {src_intf} udp-port {1}'
    }

    for key in delta:
        command = CMDS.get(key)
        if command:
            cmd = command.format(host, udp, **delta)
            commands.append(cmd)
    return commands


def main():
    argument_spec = dict(
        snmp_host=dict(required=True, type='str'),
        community=dict(type='str'),
        udp=dict(type='str', default='162'),
        version=dict(choices=['v1', 'v2c', 'v3']),
        src_intf=dict(type='str'),
        v3=dict(choices=['noauth', 'auth', 'priv']),
        vrf_filter=dict(type='str'),
        vrf=dict(type='str'),
        snmp_type=dict(choices=['trap', 'inform']),
        state=dict(choices=['absent', 'present'], default='present'),
    )

    argument_spec.update(nxos_argument_spec)

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    warnings = list()
    check_args(module, warnings)
    results = {'changed': False, 'commands': [], 'warnings': warnings}

    snmp_host = module.params['snmp_host']
    community = module.params['community']
    udp = module.params['udp']
    version = module.params['version']
    src_intf = module.params['src_intf']
    v3 = module.params['v3']
    vrf_filter = module.params['vrf_filter']
    vrf = module.params['vrf']
    snmp_type = module.params['snmp_type']
    state = module.params['state']

    existing = get_snmp_host(snmp_host, udp, module)

    if version is None:
        if existing:
            version = existing.get('version')
        else:
            version = 'v1'

    if snmp_type is None:
        if existing:
            snmp_type = existing.get('snmp_type')
        else:
            snmp_type = 'trap'

    if v3 is None:
        if version == 'v3' and existing:
            v3 = existing.get('v3')

    if snmp_type == 'inform' and version == 'v1':
        module.fail_json(msg='inform requires snmp v2c or v3')

    if (version == 'v1' or version == 'v2c') and v3:
        module.fail_json(msg='param: "v3" should not be used when '
                             'using version v1 or v2c')

    if not any([vrf_filter, vrf, src_intf]):
        if not all([snmp_type, version, community, udp]):
            module.fail_json(msg='when not configuring options like '
                                 'vrf_filter, vrf, and src_intf,'
                                 'the following params are required: '
                                 'type, version, community')

    if version == 'v3' and v3 is None:
        module.fail_json(msg='when using version=v3, the param v3 '
                             '(options: auth, noauth, priv) is also required')

    # existing returns the list of vrfs configured for a given host
    # checking to see if the proposed is in the list
    store = existing.get('vrf_filter')
    if existing and store:
        if vrf_filter not in existing['vrf_filter']:
            existing['vrf_filter'] = None
        else:
            existing['vrf_filter'] = vrf_filter
    commands = []

    args = dict(
        community=community,
        snmp_host=snmp_host,
        udp=udp,
        version=version,
        src_intf=src_intf,
        vrf_filter=vrf_filter,
        v3=v3,
        vrf=vrf,
        snmp_type=snmp_type
    )
    proposed = dict((k, v) for k, v in args.items() if v is not None)

    if state == 'absent' and existing:
        if proposed.get('community'):
            commands.append(remove_snmp_host(snmp_host, udp, existing))
        else:
            if proposed.get('src_intf'):
                commands.append(remove_src(snmp_host, udp, proposed, existing))
            if proposed.get('vrf'):
                commands.append(remove_vrf(snmp_host, udp, proposed, existing))
            if proposed.get('vrf_filter'):
                commands.append(remove_filter(snmp_host, udp, proposed, existing))

    elif state == 'present':
        delta = dict(set(proposed.items()).difference(existing.items()))
        if delta:
            command = config_snmp_host(delta, udp, proposed, existing, module)
            commands.append(command)

    cmds = flatten_list(commands)
    if cmds:
        results['changed'] = True
        if not module.check_mode:
            load_config(module, cmds)

        if 'configure' in cmds:
            cmds.pop(0)
        results['commands'] = cmds

    module.exit_json(**results)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Kevin Breit (@kbreit) <kevin.breit@kevinbreit.net>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {
    'metadata_version': '1.1',
    'status': ['preview'],
    'supported_by': 'community'
}

DOCUMENTATION = r'''
---
module: meraki_config_template
short_description: Manage configuration templates in the Meraki cloud
version_added: "2.7"
description:
- Allows for querying, deleting, binding, and unbinding of configuration templates.
notes:
- Module is not idempotent as the Meraki API is limited in what information it provides about configuration templates.
- Meraki's API does not support creating new configuration templates.
- To use the configuration template, simply pass its ID via C(net_id) parameters in Meraki modules.
options:
    state:
        description:
        - Specifies whether configuration template information should be queried, modified, or deleted.
        choices: ['absent', 'query', 'present']
        default: query
    org_name:
        description:
        - Name of organization containing the configuration template.
    org_id:
        description:
        - ID of organization associated to a configuration template.
    config_template:
        description:
        - Name of the configuration template within an organization to manipulate.
        aliases: ['name']
    net_name:
        description:
        - Name of the network to bind or unbind configuration template to.
    net_id:
        description:
        - ID of the network to bind or unbind configuration template to.
    auto_bind:
        description:
        - Optional boolean indicating whether the network's switches should automatically bind to profiles of the same model.
        - This option only affects switch networks and switch templates.
        - Auto-bind is not valid unless the switch template has at least one profile and has at most one profile per switch model.
        type: bool

author:
- Kevin Breit (@kbreit)
extends_documentation_fragment: meraki
'''

EXAMPLES = r'''
- name: Query configuration templates
  meraki_config_template:
    auth_key: abc12345
    org_name: YourOrg
    state: query
  delegate_to: localhost

- name: Bind a template from a network
  meraki_config_template:
    auth_key: abc123
    state: present
    org_name: YourOrg
    net_name: YourNet
    config_template: DevConfigTemplate
  delegate_to: localhost

- name: Unbind a template from a network
  meraki_config_template:
    auth_key: abc123
    state: absent
    org_name: YourOrg
    net_name: YourNet
    config_template: DevConfigTemplate
  delegate_to: localhost

- name: Delete a configuration template
  meraki_config_template:
    auth_key: abc123
    state: absent
    org_name: YourOrg
    config_template: DevConfigTemplate
  delegate_to: localhost
'''

RETURN = r'''
data:
    description: Information about queried object.
    returned: success
    type: complex
    contains:
        id:
          description: Unique identification number of organization
          returned: success
          type: int
          sample: L_2930418
        name:
          description: Name of configuration template
          returned: success
          type: string
          sample: YourTemplate
'''

import os
from ansible.module_utils.basic import AnsibleModule, json, env_fallback
from ansible.module_utils.urls import fetch_url
from ansible.module_utils._text import to_native
from ansible.module_utils.network.meraki.meraki import MerakiModule, meraki_argument_spec


def get_config_templates(meraki, org_id):
    path = meraki.construct_path('get_all', org_id=org_id)
    response = meraki.request(path, 'GET')
    if meraki.status != 200:
        meraki.fail_json(msg='Unable to get configuration templates')
    return response


def get_template_id(meraki, name, data):
    for template in data:
        if name == template['name']:
            return template['id']
    meraki.fail_json(msg='No configuration template named {0} found'.format(name))


def is_network_bound(meraki, nets, net_id, template_id):
    for net in nets:
        if net['id'] == net_id:
            try:
                if net['configTemplateId'] == template_id:
                    return True
            except KeyError:
                pass
    return False


def delete_template(meraki, org_id, name, data):
    template_id = get_template_id(meraki, name, data)
    path = meraki.construct_path('delete', org_id=org_id)
    path = path + '/' + template_id
    response = meraki.request(path, 'DELETE')
    if meraki.status != 200:
        meraki.fail_json(msg='Unable to remove configuration template')
    return response


def bind(meraki, org_name, net_id, nets, name, data):
    template_id = get_template_id(meraki, name, data)
    if is_network_bound(meraki, nets, net_id, template_id) is False:
        path = meraki.construct_path('bind', function='config_template', net_id=net_id)
        payload = dict()
        payload['configTemplateId'] = template_id
        if meraki.params['auto_bind']:
            payload['autoBind'] = meraki.params['auto_bind']
        meraki.result['changed'] = True
        r = meraki.request(path, method='POST', payload=json.dumps(payload))
        if meraki.status != 200:
            meraki.fail_json(msg='Unable to bind configuration template to network')
        return r


def unbind(meraki, org_name, net_id, nets, name, data):
    template_id = get_template_id(meraki, name, data)
    if is_network_bound(meraki, nets, net_id, template_id) is True:
        path = meraki.construct_path('unbind', function='config_template', net_id=net_id)
        meraki.result['changed'] = True
        r = meraki.request(path, method='POST')
        if meraki.status != 200:
            meraki.fail_json(msg='Unable to unbind configuration template from network')
        return r


def main():

    # define the available arguments/parameters that a user can pass to
    # the module
    argument_spec = meraki_argument_spec()
    argument_spec.update(state=dict(type='str', choices=['absent', 'query', 'present'], default='query'),
                         org_name=dict(type='str', aliases=['organization']),
                         org_id=dict(type='int'),
                         config_template=dict(type='str', aliases=['name']),
                         net_name=dict(type='str'),
                         net_id=dict(type='str'),
                         # config_template_id=dict(type='str', aliases=['id']),
                         auto_bind=dict(type='bool'),
                         )

    # seed the result dict in the object
    # we primarily care about changed and state
    # change is if this module effectively modified the target
    # state will include any data that you want your module to pass back
    # for consumption, for example, in a subsequent task
    result = dict(
        changed=False,
    )
    # the AnsibleModule object will be our abstraction working with Ansible
    # this includes instantiation, a couple of common attr would be the
    # args/params passed to the execution, as well as if the module
    # supports check mode
    module = AnsibleModule(argument_spec=argument_spec,
                           supports_check_mode=True,
                           )
    meraki = MerakiModule(module, function='config_template')
    meraki.params['follow_redirects'] = 'all'

    query_urls = {'config_template': '/organizations/{org_id}/configTemplates',
                  }

    delete_urls = {'config_template': '/organizations/{org_id}/configTemplates',
                   }

    bind_urls = {'config_template': '/networks/{net_id}/bind',
                 }

    unbind_urls = {'config_template': '/networks/{net_id}/unbind',
                   }

    meraki.url_catalog['get_all']['config_template'] = '/organizations/{org_id}/configTemplates'
    meraki.url_catalog['delete'] = delete_urls
    meraki.url_catalog['bind'] = bind_urls
    meraki.url_catalog['unbind'] = unbind_urls

    payload = None

    # if the user is working with this module in only check mode we do not
    # want to make any changes to the environment, just return the current
    # state with no modifications
    # FIXME: Work with Meraki so they can implement a check mode
    if module.check_mode:
        meraki.exit_json(**meraki.result)

    # execute checks for argument completeness

    # manipulate or modify the state as needed (this is going to be the
    # part where your module will do what it needs to do)
    org_id = meraki.params['org_id']
    if meraki.params['org_name']:
        org_id = meraki.get_org_id(meraki.params['org_name'])
    net_id = meraki.params['net_id']
    nets = meraki.get_nets(org_id=org_id)
    if meraki.params['net_name']:
        net_id = meraki.get_net_id(net_name=meraki.params['net_name'], data=nets)

    if meraki.params['state'] == 'query':
        meraki.result['data'] = get_config_templates(meraki, org_id)
    elif meraki.params['state'] == 'present':
        if meraki.params['net_name']:
            template_bind = bind(meraki,
                                 meraki.params['org_name'],
                                 net_id,
                                 nets,
                                 meraki.params['config_template'],
                                 get_config_templates(meraki, org_id))
    elif meraki.params['state'] == 'absent':
        if not meraki.params['net_name']:
            meraki.result['data'] = delete_template(meraki,
                                                    org_id,
                                                    meraki.params['config_template'],
                                                    get_config_templates(meraki, org_id))
        else:
            config_unbind = unbind(meraki,
                                   meraki.params['org_name'],
                                   net_id,
                                   nets,
                                   meraki.params['config_template'],
                                   get_config_templates(meraki, org_id))

    # in the event of a successful module execution, you will want to
    # simple AnsibleModule.exit_json(), passing the key/value results
    meraki.exit_json(**meraki.result)


if __name__ == '__main__':
    main()
